//====================================================================
//	Placement definition file
//--------------------------------------------------------------------
//
//	Author     : M.Frank
//====================================================================
#ifndef POOL_PLACEMENT_H
#define POOL_PLACEMENT_H

// Framework include files
#include "POOLCore/Guid.h"

// STL include files
#include <string>

/**
    POOL namespace
*/
namespace pool  {

  class Placement  {
  private:
    /// Technology identifier
    long                  m_technology;
    /// Database identifier
    std::string           m_dbID;
    /// Container identifier
    std::string           m_cntID;
    /// Object global identifier
    Guid                  m_shapeID;
  public:
    /// Standard Constructor
    Placement() : m_technology(0)             {                               }
    /// Constructor with initialization
    Placement(const std::string& db,
              const std::string& cnt,
              const Guid&        shape,
              long               tech)
    : m_technology(tech), m_dbID(db), m_cntID(cnt), m_shapeID(shape) {        }
    /// Access database identifier
    const std::string& dbID() const           {  return m_dbID;               }
    /// Set database name
    void setDb(const std::string& db)         {  m_dbID = db;                 }
    /// Access container identifier
    const std::string& contID() const         {  return m_cntID;              }
    /// Set container name
    void setCont(const std::string& cnt)      {  m_cntID = cnt;               }
    /// Access database identifier
    const Guid& shapeID() const               {  return m_shapeID;            }
    /// Access database identifier
    void setShapeID(const Guid& id)           {  m_shapeID=id;                }
    /// Access technoliogy type
    long technology() const                   {  return m_technology;         }
    /// Set technology type
    void setTechnology(long t)                {  m_technology=t; }
  };
}       // End namespace pool
#endif  // POOL_PLACEMENT_H
