#include "pool/Ref.h"
#include "pool/TestBase.h"
#include "StorageSvc/DbType.h"
#include "StorageSvc/DbInstanceCount.h"
#include "POOLCore/PoolMessageStream.h"

#include <iostream>
#include "Bug_7172.h"

static int s_printoutMod = 100;

namespace pool  {

  class Bug_7172 : public TestBase {
  public: 

    /// Standard Constructor
    Bug_7172() : TestBase(ROOTKEY_StorageType.type(), "Bug_7172_1.root", "Bug_7172_2.root")  { }

    /// Standard Destructor
    virtual ~Bug_7172()    {    }
    
    /// write objects to the persistent store
    DbStatus write()       {
      DbStatus sc;
      ConnectionH  dbcH1 = 0, dbcH2 = 0;
      sc = unlink(m_dbName2.c_str())==0 ? DbStatus::SUCCESS : DbStatus::ERROR;
      checkError(sc,"Cannot delete file: "+m_dbName2);
      sc = cache()->connect(m_dbName1, poolDb::RECREATE, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1);
      return write_objects(dbcH1, dbcH2);
    }

    DbStatus write_objects(ConnectionH dbcH1, ConnectionH dbcH2) {
      DbStatus sc;
      TransactionH trH1 = 0, trH2 = 0;
      Guid g3(getGuid("NS_Bug_7172::dummy_E"));
      Placement place1(m_dbName1,"test_cont_1", g3, m_type);
      sc = cache()->startTransaction(dbcH1, trH1);
      checkError(sc,"Cannot start database transactions (1)");
      for ( size_t i = 0; i < m_innerLoops; ++i )   {
        // Fill and write out nested collections
        pool::Ref<NS_Bug_7172::dummy_E> r(new NS_Bug_7172::dummy_E());
        r->dummy.push_back(new NS_Bug_7172::dummy_D()); 
        r->dummy.push_back(new NS_Bug_7172::dummy_D());
        r->info = new NS_Bug_7172::dummy_D();
        sc = cache()->registerForWrite(trH1, place1, r.ptr());
        checkError(sc,"Error cache.registerForWrite(trH1, place1, NS_Bug_7172::dummy_E) [1]");
      }
      std::cout << "Commit transaction..." << std::endl;
      //gDebug = 3;
      sc = cache()->commit(trH1);
      checkError(sc,"Error cache.commit(...) [1]");
      std::cout << "Clear cache..." << std::endl;
      sc = cache()->clear();
      checkError(sc,"Error cache.clear(...)");
      std::cout << "Diconnect from database accesses..." << std::endl;
      sc = cache()->disconnect(dbcH1);
      checkError(sc,"Error cache.disconnect(...) [1]");
      return sc;
    }

    /// read objects from the persistent store
    DbStatus read()  {
      DbStatus sc;
      Guid g3(getGuid("NS_Bug_7172::dummy_E"));
      ConnectionH  dbcH1 = 0, dbcH2 = 0;
      sc = cache()->connect(m_dbName1, poolDb::READ, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1);
      for(size_t i=0; i < m_innerLoops; ++i)   {
        Token *tok2 = new Token();

        // Since this example does not use collections,
        // we do it here by faking the tokens to be 
        // returned by the collections
        // 
        tok2->setDb(m_dbName1);
        tok2->setCont("test_cont_1");
        tok2->setTechnology(m_type);
        tok2->setClassID(g3);
        tok2->oid().first  = 3;
        tok2->oid().second = i;
        // Read in nested collections
        pool::Ref<NS_Bug_7172::dummy_E> _r(cache(), tok2);
        if ( _r && i%s_printoutMod == 0 )   {
          std::cout << "Can read object!!!" 
                    << typeid(*_r.ptr()).name()
                    << "  " << _r->dummy.size() 
                    << " Info:" << (unsigned long)_r->info

                    << std::endl;
        }
        else if ( i%s_printoutMod == 0 )   {
          std::cout << "NS_Bug_7172::dummy_E MISSING" << std::endl;
        }
      }
      return poolDb::Success;
    }
  };
}
pool::TestRunner<pool::Bug_7172> s_testBug_7172("Bug_7172");
