#include "pool/DataReferenceHandler.h"
#include "pool/IDataSvc.h"
#include "pool/Ref.h"
using namespace pool;

/// Standard Constructor
DataReferenceHandler::DataReferenceHandler(IDataSvc* cache) : m_cache(cache)  {
}

/// Standard Destructor
DataReferenceHandler::~DataReferenceHandler()   {
}

/// Convert transient reference to persistent OID
DbStatus DataReferenceHandler::saveReference(Reference* pRef, Token::OID_t* data) {
  RefBase* r = (RefBase*)pRef;
  Token* tok = r->token();
  if ( r->object() && !tok ) {
    DbStatus sc = m_cache->findToken(r->object(), tok);
    if ( !sc.isSuccess() )  {
      // Serious error, but unrecoverable; unclear what to do
      // This reference can never ever be followed when reading.
      std::cout << "Serious error while reference conversion." << std::endl;
      return sc;
    }
    //std::cout << "ConvToken:" << tok->toString() << std::endl;
    r->setToken(tok);
    r->setDataSvc(m_cache);
    return saveToken(tok, data);
  }
  else if ( tok )  {
    //std::cout << "ConvToken:" << tok->toString() << std::endl;
    r->setDataSvc(m_cache);
    return saveToken(tok, data);
  }
  return Success;
}

/// Convert persistent OID into a transient reference
DbStatus DataReferenceHandler::loadReference(Reference* pRef, Token::OID_t* data) {
  Token *tok = 0, *pTok = 0;
  RefBase* r = (RefBase*)pRef;
  DbStatus sc = createToken(data, tok);
  if ( sc.isSuccess() )   {
    sc = m_cache->regToken(tok, pTok);
    if ( sc.isSuccess() )   {
      r->setToken(pTok);
    }
    tok->release();
  }
  r->setDataSvc(m_cache);
  //Token* tt = r->token();
  //if ( !tt ) std::cout << "Invalid Token!" << std::endl;
  //else std::cout << "Valid token:" << tt->toString() << std::endl;
  return sc;
}
