#include "pool/TestBase.h"
#include "StorageSvc/DbType.h"
#include "StorageSvc/FileDescriptor.h"
#include "ObjyMemoryFile.h"
#include "Test0.h"
#include "TNamed.h"

#include "TROOT.h"
#include "TClass.h"
#include "TStreamerInfo.h"
#include "TStreamerElement.h"

static int s_printoutMod = 100;

namespace pool  {

  class CMSTest1 : public TestBase {
  public: 

    /// Standard Constructor
    CMSTest1() : TestBase(ROOTKEY_StorageType.type(), "CMS_Test1.root", "CMS_Test2.root") 
    {   }

    /// Standard Destructor
    virtual ~CMSTest1()    {    }
    
    /// write objects to the persistent store
    DbStatus write()       {
      DbStatus sc;
      ConnectionH  dbcH1 = 0, dbcH2 = 0;
      TransactionH trH1 = 0, trH2 = 0;

      Guid g1(getGuid("Test1::C<int>"));
      Placement place1(m_dbName1,"test_cont_1", g1, ROOTKEY_StorageType.type());

      sc = cache()->connect(m_dbName1, poolDb::RECREATE, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1);

      for ( size_t j = 0; j < m_outerLoops; ++j)  {
        sc = cache()->startTransaction(dbcH1, trH1);
        checkError(sc,"Cannot start database transactions (1)");
        for ( size_t i = 0; i < m_innerLoops; ++i )   {
          pool::Ref<Test1::C<int>  > refFile(new Test1::C<int>());
          sc = cache()->registerForWrite(trH1, place1, refFile.ptr());
          if ( !sc.isSuccess() )   {
            checkError(sc,"Error cache.registerForWrite(trH1, place1, refFile) [1]");
          }
        }
        std::cout << "Commit transaction..." << std::endl;
        sc = cache()->commit(trH1);

        checkError(sc,"Error cache.commit(...) [1]");
        std::cout << "Diconnect from database accesses..." << std::endl;
        sc = cache()->disconnect(0);
        checkError(sc,"Error cache.disconnect(...) [1]");
        std::cout << "Clear cache..." << std::endl;
        sc = cache()->clear();
        checkError(sc,"Error cache.clear(...)");
        sc = cache()->connect(m_dbName1, poolDb::UPDATE, dbcH1);
        checkError(sc,"Cannot connect to database "+m_dbName1);
      }
      sc = cache()->disconnect(0);
      dumpStreamerInfo("Test1::A");
      dumpStreamerInfo("Test1::B");
      dumpStreamerInfo("Test1::C<int>");
      return sc;
    }
    /// write objects to the persistent store
    DbStatus update()       {
      DbStatus sc;
      ConnectionH  dbcH1 = 0, dbcH2 = 0;
      TransactionH trH1 = 0, trH2 = 0;

      Guid g1(getGuid("Test1::C<int>"));
      Placement place1(m_dbName1,"test_cont_1", g1, ROOTKEY_StorageType.type());


      for ( size_t j = 0; j < m_outerLoops; ++j)  {
        sc = cache()->connect(m_dbName1, poolDb::READ, dbcH1);
        checkError(sc,"Cannot connect to database "+m_dbName1);

        FileDescriptor dsc1(m_dbName1, m_dbName1);
        dsc1.setDbc(dbcH1);

        sc = cache()->startTransaction(dbcH1, trH1);
        checkError(sc,"Cannot start database transactions (1)");
        std::cout << "Rollback transaction..." << std::endl;
        sc = cache()->rollback(trH1);
        sc = storageSvc()->reconnect(dsc1, poolDb::UPDATE);
        sc = cache()->startTransaction(dbcH1, trH1);
        checkError(sc,"Cannot start database transactions (1)");

        for ( size_t i = 0; i < m_innerLoops; ++i )   {
          pool::Ref<Test1::C<int>  > refFile(new Test1::C<int>());
          sc = cache()->registerForWrite(trH1, place1, refFile.ptr());
          if ( !sc.isSuccess() )   {
            checkError(sc,"Error cache.registerForWrite(trH1, place1, refFile) [1]");
          }
        }
        std::cout << "Commit transaction..." << std::endl;
        sc = cache()->commit(trH1);

        checkError(sc,"Error cache.commit(...) [1]");
        std::cout << "Diconnect from database accesses..." << std::endl;
        sc = cache()->disconnect(0);
        checkError(sc,"Error cache.disconnect(...) [1]");
        std::cout << "Clear cache..." << std::endl;
        sc = cache()->clear();
        checkError(sc,"Error cache.clear(...)");
      }
      sc = cache()->disconnect(0);
      dumpStreamerInfo("Test1::A");
      dumpStreamerInfo("Test1::B");
      dumpStreamerInfo("Test1::C<int>");
      return sc;
    }
  };
}

pool::TestRunner<pool::CMSTest1> s_testCMSTest1("CMSTest1");
