#include "pool/TestBase.h"
#include "StorageSvc/DbType.h"
#include "StorageSvc/IStorageSvc.h"
#include "StorageSvc/FileDescriptor.h"
#include "ObjyMemoryFile.h"
#include "ExampleClass.h"

#include "TROOT.h"
#include "TClass.h"
#include "TStreamerInfo.h"
#include "TStreamerElement.h"

static int s_printoutMod = 100;

namespace pool  {

  class CMSTest2 : public TestBase {
  public: 
    Guid g1;

    /// Standard Constructor
    CMSTest2() : TestBase(ROOTKEY_StorageType.type(), "CMS_Test2_1.root", "CMS_Test2_2.root") 
    {   
      g1.Data1 = 0x4FFF4DBB;
      g1.Data2 = 0xFFE2;
      g1.Data3 = 0x4F0D;
      *(int*)&g1.Data4[0] = 0x95D72047;
      *(int*)&g1.Data4[4] = 0x37331AD5;
    }

    /// Standard Destructor
    virtual ~CMSTest2()    {    }
    
    /// write objects to the persistent store
    DbStatus write()       {
      DbStatus sc;
      TransactionH trH1 = 0;
      ConnectionH  dbcH1 = 0;
      Placement place1(m_dbName1,"test_cont_1", g1, ROOTKEY_StorageType.type());
      for ( size_t j = 0; j < m_outerLoops; ++j)  {
        sc = cache()->connect(m_dbName1, poolDb::RECREATE, dbcH1);
        checkError(sc,"Cannot connect to database "+m_dbName1);
        sc = cache()->startTransaction(dbcH1, trH1);
        checkError(sc,"Cannot start database transactions (1)");
        for ( size_t i = 0; i < m_innerLoops; ++i )   {
          pool::Ref<ExampleClass> refH(new ExampleClass());
          sc = cache()->registerForWrite(trH1, place1, refH.ptr());
          refH->cont.setStatus(i);
          refH->cont.setId(10*i);
          refH->cont.setLostHits(100*i);
          if ( !sc.isSuccess() )   {
            checkError(sc,"Error cache.registerForWrite(trH1, place1, refH) [1]");
          }
        }
        std::cout << "Commit transaction..." << std::endl;
        sc = cache()->commit(trH1);
        checkError(sc,"Error cache.commit(...) [1]");
        std::cout << "Diconnect from database accesses..." << std::endl;
        sc = cache()->disconnect(0);
        checkError(sc,"Error cache.disconnect(...) [1]");
        std::cout << "Clear cache..." << std::endl;
        sc = cache()->clear();
        checkError(sc,"Error cache.clear(...)");
      }
      dumpStreamerInfo("ExampleClass");
      dumpStreamerInfo("TTrack");
      dumpStreamerInfo("RecObj");
      dumpStreamerInfo("PRecTrack");
      return sc;
    }
    /// write objects to the persistent store
    DbStatus read()       {
      DbStatus sc;
      TransactionH trH1 = 0;
      ConnectionH  dbcH1 = 0;

      //Guid g1(getGuid(std::string("ExampleClass")));
      Placement place1(m_dbName1,"test_cont_1", g1, ROOTKEY_StorageType.type());
      for ( size_t j = 0; j < m_outerLoops; ++j)  {
        sc = cache()->connect(m_dbName1, poolDb::READ, dbcH1);
        checkError(sc,"Cannot connect to database "+m_dbName1);
        for (size_t i = 0; i < m_innerLoops; ++i )   {
          Token *tok1 = new Token();
          tok1->setDb(m_dbName1);
          tok1->setCont("test_cont_1");
          tok1->setTechnology(ROOTKEY_StorageType.type());
          tok1->setClassID(g1);
          tok1->oid().first  = poolDb::INVALID;
          tok1->oid().second = i;

          if ( i % s_printoutMod == 0 ) {
            std::cout << "======================= New round....=====================" << std::endl;
          }

          pool::Ref<ExampleClass> refH(cache(), tok1);
          if ( refH && i%s_printoutMod == 0 )   {
            std::cout << "Status:" << refH->cont.status()
                      << " ID:" << refH->cont.id() << " lost:" << refH->cont.lostHits() 
                      << std::endl;
          }
        }
        cache()->clear();
        std::cout << "Clear cache..." << std::endl;
        sc = cache()->disconnect(dbcH1);
      }
      dumpStreamerInfo("ExampleClass");
      dumpStreamerInfo("TTrack");
      dumpStreamerInfo("RecObj");
      dumpStreamerInfo("PRecTrack");
      return sc;
    }
  };
}

pool::TestRunner<pool::CMSTest2> s_testCMSTest2("CMSTest2");
