#include "pool/test_bool.h"
#include "pool/TestBase.h"
#include "pool/Ref.h"
#include "StorageSvc/DbType.h"
#include "StorageSvc/DbInstanceCount.h"

static int s_printoutMod = 100;

namespace pool  {

  class Booleans : public TestBase {
  public: 

    /// Standard Constructor
    Booleans() : TestBase(ROOTKEY_StorageType.type(), "test_Booleans.root", "None") 
    {
    }

    /// Standard Destructor
    virtual ~Booleans()    {    }
    
    /// write objects to the persistent store
    DbStatus write()       {
      ConnectionH  dbcH1 = 0;
      DbStatus sc = cache()->connect(m_dbName1, poolDb::RECREATE, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1);

      TransactionH trH1 = 0;
      Placement place1(m_dbName1,"test_cont_1", getGuid("test_bool"), m_type);

      for ( size_t j = 0; j < m_outerLoops; ++j )   {
        sc = cache()->startTransaction(dbcH1, trH1);
        checkError(sc,"Cannot start database transactions (1)");
        for ( size_t i = 0; i < m_innerLoops; ++i )   {
          pool::Ref<test_bool>   refObj(new test_bool);
          refObj->fillData(i);

          sc = cache()->registerForWrite(trH1, place1, refObj.ptr());
          checkError(sc,"Error cache.registerForWrite(trH1, place1, test)   [2]");

        }
        std::cout << "Commit transaction..." << std::endl;

        sc = cache()->commit(trH1);
        checkError(sc,"Error cache.commit(...) [1]");
        std::cout << "Clear cache..." << std::endl;
        sc = cache()->clear();
        checkError(sc,"Error cache.clear(...)");
      }
      dumpStreamerInfo("test_bool");
      std::cout << "Diconnect from database accesses..." << std::endl;
      sc = cache()->disconnect(0);
      checkError(sc,"Error cache.disconnect(...) [1]");
      return sc;
    }

    /// read objects from the persistent store
    DbStatus read()        {
      ConnectionH  dbcH1 = 0;
      DbStatus sc = cache()->connect(m_dbName1, poolDb::READ, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1);

      for (size_t i = 0; i < m_innerLoops; ++i )   {
        Token *tok2 = new Token();
        tok2->setDb(m_dbName1);
        tok2->setCont("test_cont_1");
        tok2->setTechnology(m_type);
        tok2->setClassID(getGuid("test_bool"));
        tok2->oid().first  = poolDb::INVALID;
        tok2->oid().second = i;

        // Read first object
        pool::Ref<test_bool> refObj(cache(), tok2);
        if ( refObj && i%s_printoutMod == 0 )   {
          refObj->printData(i);
        }
      }
      cache()->clear();
      std::cout << "Clear cache..." << std::endl;
      sc = cache()->disconnect(0);
      checkError(sc,"cache.disconnect(0)");
      return sc;
    }

  };
}

pool::TestRunner<pool::Booleans> s_testBooleans("Booleans");
