#include "pool/test.h"
#include "pool/TestBase.h"
#include "StorageSvc/DbType.h"
#include "StorageSvc/DbPrint.h"
#include "StorageSvc/DbReflex.h"
#include "StorageSvc/DbInstanceCount.h"

static int s_printoutMod = 100;

namespace pool  {

  class DataTypes : public TestBase {
  public: 

    /// Standard Constructor
    DataTypes() : TestBase(ROOTKEY_StorageType.type(), "test_1_1.root", "test_1_2.root")  { }

    /// Standard Destructor
    virtual ~DataTypes()    {    }
    
    /// write objects to the persistent store
    DbStatus write()       {
      DbStatus sc;
      ConnectionH  dbcH1 = 0, dbcH2 = 0;
#define __w
#if defined(__w)
      sc = unlink(m_dbName2.c_str())==0 ? DbStatus::Success : DbStatus::Error;
      checkError(sc,"Cannot delete file: "+m_dbName2);
      sc = cache()->connect(m_dbName1, poolDb::RECREATE, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1, true);
      sc = cache()->connect(m_dbName2, poolDb::CREATE, dbcH2);
      checkError(sc,"Cannot connect to database "+m_dbName2, true);
      return write_objects(dbcH1, dbcH2);
#else
      sc = cache()->connect(m_dbName1, poolDb::UPDATE, dbcH1);
      checkError(sc,"Cannot reconnect to database in UPDATE mode.", true);
      sc = cache()->connect(m_dbName2, poolDb::UPDATE, dbcH2);
      checkError(sc,"Cannot reconnect to database in UPDATE mode.", true);
      return write_objects(dbcH1, dbcH2);
#endif
    }

    /// write objects to the persistent store
    DbStatus update()       {
      DbStatus sc;
      ConnectionH  dbcH1 = 0, dbcH2 = 0;
#if 0
      sc = cache()->connect(m_dbName1, poolDb::UPDATE, dbcH1);
      checkError(sc,"Cannot reconnect to database in UPDATE mode.", true);
      sc = cache()->connect(m_dbName2, poolDb::UPDATE, dbcH2);
      checkError(sc,"Cannot reconnect to database in UPDATE mode.", true);
      return write_objects(dbcH1, dbcH2);
#endif
      sc = cache()->connect(m_dbName1, poolDb::UPDATE, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1, true);
      sc = cache()->connect(m_dbName2, poolDb::UPDATE, dbcH2);
      checkError(sc,"Cannot connect to database "+m_dbName2, true);
      //return write_objects(dbcH1, dbcH2);
      return update_objects();
    }

    DbStatus write_objects(ConnectionH dbcH1, ConnectionH dbcH2)
    {
      DbStatus sc;
      TransactionH trH1 = 0, trH2 = 0;
      Guid g3(getGuid("AnotherTNamed"));
      Guid g4(getGuid("A::B::C::Named"));
      Guid g5(getGuid("DumbStruct<int>"));
      Guid g6(getGuid("EventHeader"));

      Placement place1(m_dbName1,"test_cont_1", getGuid("TNamed"), ROOTKEY_StorageType.type());
      Placement place2(m_dbName2,"test_cont_2", getGuid("test"), m_type);
      Placement place31(m_dbName1,"test_cont_1", g3, ROOTKEY_StorageType.type());
      Placement place32(m_dbName2,"test_another_cont", g3, ROOTKEY_StorageType.type());
      Placement place4(m_dbName1,"test_cont_1", g4, ROOTKEY_StorageType.type());
      Placement place5(m_dbName1,"test_cont_3", g5, m_type);
      Placement place6(m_dbName1,"test_cont_4", g6, m_type);

      accessRootClass("test");
      accessRootClass("AnotherTNamed");
      accessRootClass("pool::Ref<test>");
      accessRootClass("pool::Ref<TNamed>");
      dumpStreamerInfo("AnotherTNamed");

      for ( size_t j = 0; j < m_outerLoops; ++j )   {
        sc = cache()->startTransaction(dbcH1, trH1);
        checkError(sc,"Cannot start database transactions (1)", true);
        sc = cache()->startTransaction(dbcH2, trH2);
        checkError(sc,"Cannot start database transactions (2)", true);
        for ( size_t i = 0; i < m_innerLoops; ++i )   {
          char txt1[256], txt2[256], txt3[256], txt4[256], txt5[256];
          ::sprintf(txt1,"Hello [TNamed(%d)]", int(i));
          ::sprintf(txt2,"World [TNamed(%d)]", int(i));
          ::sprintf(txt3,"Hello [test(%d)]", int(i));
          ::sprintf(txt4,"AnotherTNamed_%d", int(i));
          ::sprintf(txt5,"A::B::C::Named_%d", int(i));
          printRound(i, s_printoutMod);
          pool::Ref<A::B::C::Named> refNS(new A::B::C::Named(txt5, txt3));
          pool::Ref<AnotherTNamed> refAnother(new AnotherTNamed(txt4, txt3));
          pool::Ref<TNamed> refNamed1(new TNamed(txt1, txt2));
          pool::Ref<test>   refObj(new test);
          pool::Ref<DumbStruct<int> > refDumb(new DumbStruct<int>(i));
          pool::Ref<EventHeader > refEVH(new EventHeader());

          refEVH->addElement( EventHeaderElement(1,"one","one"));
          refEVH->addElement( EventHeaderElement(2,"two","two"));
          refEVH->addElement( EventHeaderElement(3,"three","three"));

          refAnother->str = "For the sake of it!";
          refNS->str      = "Namespaces are fun!";

          refObj->setTNamed1(refNamed1);    // Assign object pointer to reference
          refObj->setTNamed2(refAnother);   // Assign object pointer to reference

          refObj->fillData(i);
          refAnother->refTest = refObj;

          sc = cache()->registerForWrite(trH1, place1, refNamed1.ptr());
          checkError(sc,"Error cache.registerForWrite(trH1, place1, TNamed) [1]", true);

          sc = cache()->registerForWrite(trH2, place2, refObj.ptr());
          checkError(sc,"Error cache.registerForWrite(trH2, place2, test)   [2]", true);

          sc = cache()->registerForWrite(trH1, place31, refAnother.ptr());
          checkError(sc,"Error cache.registerForWrite(trH1, place3, AnotherTNamed)   [3]", true);

          //sc = cache()->registerForWrite(trH2, place32, refAnother.ptr());
          //checkError(sc,"Error cache.registerForWrite(trH1, place3, AnotherTNamed)   [3]", true);

          sc = cache()->registerForWrite(trH1, place4, refNS.ptr());
          checkError(sc,"Error cache.registerForWrite(trH1, place4, A::B::C::Named) [4]", true);

          sc = cache()->registerForWrite(trH1, place5, refDumb.ptr());
          checkError(sc,"Error cache.registerForWrite(trH1, place5, DumbStruct<int>) [5]", true);

          sc = cache()->registerForWrite(trH1, place6, refEVH.ptr());
          checkError(sc,"Error cache.registerForWrite(trH1, place6, EventHeader) [6]", true);
          if ( refObj && i%s_printoutMod == 0 )   {
            refObj->printData(i, false);
          }
        }
        std::cout << "Commit transaction..." << std::endl;

        //gDebug = 3;
        sc = cache()->commit(trH2);
        checkError(sc,"Error cache.commit(...) [2]", true);
        sc = cache()->commit(trH1);
        checkError(sc,"Error cache.commit(...) [1]", true);
        std::cout << "Clear cache..." << std::endl;
        sc = cache()->clear();
        checkError(sc,"Error cache.clear(...)");
      }
#if 0
      dumpClass("AnotherTNamed");
      dumpClass("test_base_base");
      dumpClass("test_base");
      dumpClass("test");
      dumpClass("test_base::EmbeddedClasses");
      dumpClass("test_base::EmbeddedClasses::Embedded1");
      dumpClass("test_base::EmbeddedClasses::Embedded2");
      dumpClass("test_base::EmbeddedClasses::Embedded3");
#endif
      std::cout << "Diconnect from database accesses..." << std::endl;
      sc = cache()->disconnect(dbcH1);
      checkError(sc,"Error cache.disconnect(...) [1]", true);
      sc = cache()->disconnect(dbcH2);
      checkError(sc,"Error cache.disconnect(...) [2]", true);
      return sc;
    }
    /// read objects from the persistent store
    DbStatus read()        {
      DbStatus sc;
      Guid g3(getGuid("AnotherTNamed"));
      Guid g4(getGuid("A::B::C::Named"));
      Guid g5(getGuid("DumbStruct<int>"));
      Guid g6(getGuid("EventHeader"));
      TransactionH trH1 = 0, trH2 = 0;
      ConnectionH  dbcH1 = 0, dbcH2 = 0;

      dumpStreamerInfo("AbstractImp");
      dumpStreamerInfo("AbstractBase");
      accessRootClass("test");
      accessRootClass("AnotherTNamed");
      accessRootClass("pool::Ref<test>");
      accessRootClass("pool::Ref<TNamed>");
      dumpStreamerInfo("AnotherTNamed");

      sc = cache()->connect(m_dbName1, poolDb::READ, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1, true);
      sc = cache()->connect(m_dbName2, poolDb::READ, dbcH2);
      checkError(sc,"Cannot connect to database "+m_dbName2, true);
      sc = cache()->startTransaction(dbcH1, trH1);
      checkError(sc,"Cannot start database transactions (1)", true);
      sc = cache()->startTransaction(dbcH2, trH2);
      checkError(sc,"Cannot start database transactions (2)", true);

      ClassH cl = DbReflex::forTypeName("EventHeaderBase");
      std::cout << "Abstract:" << (char*)(cl.IsAbstract() ? "Yes" : "No") << std::endl;
      std::cout << "Virtual: " << (char*)(cl.IsVirtual()  ? "Yes" : "No") << std::endl;

      for (size_t i = 0; i < m_innerLoops; ++i )   {
        Token *tok2 = new Token();

        // Since this example does not use collections,
        // we do it here by faking the tokens to be 
        // returned by the collections
        // 
        tok2->setDb(m_dbName2);
        tok2->setCont("test_cont_2");
        tok2->setTechnology(m_type);
        tok2->setClassID(getGuid("test"));
        tok2->oid().first  = 3;
        tok2->oid().second = i;

        Token *tok3 = new Token();
        tok3->setDb(m_dbName1);
        tok3->setCont("test_cont_1");
        tok3->setTechnology(ROOTKEY_StorageType.type());
        tok3->setClassID(g3);
        tok3->oid().first  = 4;
        tok3->oid().second = (3*i)+1;

        Token *tok4 = new Token();
        tok4->setDb(m_dbName1);
        tok4->setCont("test_cont_1");
        tok4->setTechnology(ROOTKEY_StorageType.type());
        tok4->setClassID(g4);
        tok4->oid().first  = 5;
        tok4->oid().second = (3*i)+2;

        Token *tok5 = new Token();
        tok5->setDb(m_dbName1);
        tok5->setCont("test_cont_3");
        tok5->setTechnology(m_type);
        tok5->setClassID(g5);
        tok5->oid().first  = 6;
        tok5->oid().second = i;

        Token *tok6 = new Token();
        tok6->setDb(m_dbName1);
        tok6->setCont("test_cont_4");
        tok6->setTechnology(m_type);
        tok6->setClassID(g6);
        tok6->oid().first  = 7;
        tok6->oid().second = i;

        printRound(i, s_printoutMod);
        DbPrint log("TestDataTypes");
        log << DbPrintLvl::Info;

        // Read first object
        pool::Ref<test> refObj(cache(), tok2);
        if ( refObj ) {
          tok2 = tok2;
        }
        pool::Ref<AnotherTNamed> refOtherTNamed(cache(), tok3);
        //if ( refOtherTNamed )   {
        if ( refOtherTNamed && i%s_printoutMod == 0 )   {
          std::cout << "Mixed Keyed containers work:" 
                    << refOtherTNamed->GetName() << " -- "
                    << refOtherTNamed->GetTitle() << " str:"
                    << refOtherTNamed->str << " typid:"
                    << typeid(*refOtherTNamed.ptr()).name()
                    << std::endl;
          Token* t = refOtherTNamed.token();
          std::string val = refOtherTNamed.token()->toString();
          std::cout << "Token [AnotherTNamed]      :" << (void*)t << " " << val << std::endl;
          t = refOtherTNamed->refTest.token();
          val = t ? t->toString() : std::string("INVALID TOKEN");
          std::cout << "Token [AnotherTNamed->test]:" << (void*)t << " " << val << std::endl;
        }
        else if ( i%s_printoutMod == 0 )   {
          std::cout << "AnotherTNamed MISSING" << std::endl;
        }
        pool::Ref<A::B::C::Named> refNS(cache(), tok4);
        if ( refNS && i%s_printoutMod == 0 )   {
          std::cout << "Namespaces work:" 
                    << refNS->GetName() << " -- "
                    << refNS->GetTitle() << " str:"
                    << refNS->str << " typid:"
                    << typeid(*refNS.ptr()).name() << std::endl;
          log << (void*)refNS.token() << " " << refNS.token()->toString() << std::endl;
        }
        else if ( i%s_printoutMod == 0 )   {
          std::cout << "A::B::C::Named MISSING";
        }

        pool::Ref<DumbStruct<int> > refDumb(cache(), tok5);
        if ( refDumb && i%s_printoutMod == 0 )   {
          std::cout << " Dumb:" << refDumb->i;
        }
        else if ( i%s_printoutMod == 0 )   {
          std::cout << "AnotherTNamed MISSING";
        }

        pool::Ref<EventHeader> refEVH(cache(), tok6);
        if ( refEVH && i%s_printoutMod == 0 )   {
          std::cout << " refEVH valid";
        }
        else if ( i%s_printoutMod == 0 )   {
          std::cout << "EventHeader MISSING";
        }

        if ( i%s_printoutMod == 0 )   {
          std::cout << std::endl;
        }

        // Read first object
        //pool::Ref<test> refObj(cache(), tok2);
#if 0
        dumpStreamerInfo("test_base_base");
        dumpStreamerInfo("test_base");
        dumpStreamerInfo("test");
        gDebug = 2;
#endif
        if ( refObj && i%s_printoutMod == 0 )   {
          refObj->printData(i,true);
        }
        else if ( i%s_printoutMod == 0 )   {
          std::cout << "test MISSING";
        }
        cache()->clear();
      }
      sc = cache()->commit(trH2);
      checkError(sc,"Error cache.commit(...) [2]", true);
      sc = cache()->commit(trH1);
      checkError(sc,"Error cache.commit(...) [1]", true);
      std::cout << "Clear cache..." << std::endl;
      cache()->clear();
      sc = cache()->disconnect(0);
      checkError(sc,"cache.disconnect(0)");
      return sc;
    }
    /// Update objects from the persistent store
    DbStatus update_objects()  {
      DbStatus sc;
      TransactionH trH1 = 0;
      TransactionH trH2 = 0;
      Guid g3(getGuid("AnotherTNamed"));
      ConnectionH  dbcH1 = 0, dbcH2 = 0;
      sc = cache()->connect(m_dbName1, poolDb::READ, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1, true);
      sc = cache()->connect(m_dbName2, poolDb::READ, dbcH2);
      checkError(sc,"Cannot connect to database "+m_dbName2, true);

      sc = cache()->reconnect(dbcH1, poolDb::UPDATE);
      checkError(sc,"Cannot reconnect to database in UPDATE mode.", true);
      sc = cache()->startTransaction(dbcH1, trH1);
      checkError(sc,"Cannot start database transactions (1)", true);

      sc = cache()->reconnect(dbcH2, poolDb::UPDATE);
      checkError(sc,"Cannot reconnect to database in UPDATE mode.", true);
      sc = cache()->startTransaction(dbcH2, trH2);
      checkError(sc,"Cannot start database transactions (2)", true);

      accessRootClass("test"); 
      accessRootClass("AnotherTNamed");
      accessRootClass("pool::Ref<test>");
      accessRootClass("pool::Ref<TNamed>");

      for (size_t i = 0; i < m_innerLoops; ++i )   {
        Token *tok2 = new Token();
        // Since this example does not use collections,
        // we do it here by faking the tokens to be 
        // returned by the collections
        // 
        tok2->setDb(m_dbName2);
        tok2->setCont("test_cont_2");
        tok2->setTechnology(m_type);
        tok2->setClassID(getGuid("test"));
        tok2->oid().first  = poolDb::INVALID;
        tok2->oid().second = i;

        Token *tok3 = new Token();
        tok3->setDb(m_dbName1);
        tok3->setCont("test_cont_1");
        tok3->setTechnology(ROOTKEY_StorageType.type());
        tok3->setClassID(g3);
        tok3->oid().first  = poolDb::INVALID;
        tok3->oid().second = (3*i)+1;

        printRound(i, s_printoutMod);

        pool::Ref<AnotherTNamed> refOtherTNamed(cache(), tok3);
        if ( refOtherTNamed && i%s_printoutMod == 0 )   {
          std::cout << "Mixed Keyed containers work:" 
                    << refOtherTNamed->GetName() << " -- "
                    << refOtherTNamed->GetTitle() << " str:"
                    << refOtherTNamed->str
                    << std::endl;
        }
        if ( refOtherTNamed ) {
          std::string tmp = refOtherTNamed->GetName();
          tmp += " - Modified";
          refOtherTNamed->SetName(tmp.c_str());
          sc = cache()->update(trH1, refOtherTNamed.ptr());
          checkError(sc,"cache.update(refOtherTNamed)", true);
        }
        // Read first object
        pool::Ref<test> refObj(cache(), tok2);
        if ( refObj )   {
          if ( (i % 3) == 0 ) {
            refObj->m_string += " - Modified";
            sc = cache()->update(trH2, refObj.ptr());
          }
          if ( i % s_printoutMod == 0 ) {
            refObj->printData(i,false);
          }
        }
      }
      sc = cache()->commit(trH1);
      checkError(sc,"Error cache.commit(...) [1]", true);
      sc = cache()->commit(trH2);
      checkError(sc,"Error cache.commit(...) [2]", true);
      cache()->clear();
      std::cout << "Clear cache..." << std::endl;
      sc = cache()->disconnect(0);
      checkError(sc,"cache.disconnect(0)");
      return sc;
    }

    /// read objects from the persistent store and destroy them
    DbStatus destroy()  {
      DbStatus sc;
      TransactionH trH1 = 0;
      Guid g3(getGuid("AnotherTNamed"));
      ConnectionH  dbcH1 = 0, dbcH2 = 0;
      sc = cache()->connect(m_dbName1, poolDb::UPDATE, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1);
      sc = cache()->connect(m_dbName2, poolDb::READ, dbcH2);
      checkError(sc,"Cannot connect to database "+m_dbName2);

      for (size_t i = 0; i < m_innerLoops; ++i )   {
        Token *tok2 = new Token();
        // Since this example does not use collections,
        // we do it here by faking the tokens to be 
        // returned by the collections
        // 
        tok2->setDb(m_dbName2);
        tok2->setCont("test_cont_2");
        tok2->setTechnology(m_type);
        tok2->setClassID(getGuid("test"));
        tok2->oid().first  = poolDb::INVALID;
        tok2->oid().second = i;

        Token *tok3 = new Token();
        tok3->setDb(m_dbName1);
        tok3->setCont("test_cont_1");
        tok3->setTechnology(ROOTKEY_StorageType.type());
        tok3->setClassID(g3);
        tok3->oid().first  = 2;
        tok3->oid().second = (3*i)+1;

        printRound(i, s_printoutMod);

        pool::Ref<AnotherTNamed> refOtherTNamed(cache(), tok3);
        if ( refOtherTNamed && i%s_printoutMod == 0 )   {
          std::cout << "Mixed Keyed containers work:" 
                    << refOtherTNamed->GetName() << " -- "
                    << refOtherTNamed->GetTitle() << " str:"
                    << refOtherTNamed->str
                    << std::endl;
        }
        if ( refOtherTNamed ) {
          sc = cache()->startTransaction(dbcH1, trH1);
          checkError(sc,"Cannot start database transactions (1)");
          sc = cache()->destroy(trH1, refOtherTNamed.ptr());
          checkError(sc,"cache.update(refOtherTNamed)");
          sc = cache()->commit(trH1);
          checkError(sc,"Error cache.commit(...) [1]");

        }
        // Read first object
        pool::Ref<test> refObj(cache(), tok2);
        if ( refObj && i%s_printoutMod == 0 )   {
          refObj->printData(i,true);
        }
        cache()->clear();
      }
      std::cout << "Clear cache..." << std::endl;
      sc = cache()->disconnect(0);
      checkError(sc,"cache.disconnect(0)");
      return sc;
    }

  };
}

pool::TestRunner<pool::DataTypes> s_testDataTypes("DataTypes");
