#include "pool/test.h"
#include "pool/TestBase.h"
#include "StorageSvc/DbType.h"

static int s_printoutMod = 100;
namespace pool  {

  class FileAging : public TestBase {
  public: 

    /// Standard Constructor
    FileAging() : TestBase(ROOTKEY_StorageType.type(), "test_multi_db", "multi_db_test_X")
    {  m_innerLoops = 400;  }

    /// Standard Destructor
    virtual ~FileAging()    {    }

    /// write objects to the persistent store
    DbStatus write()       {
      size_t i, j;
      DbStatus sc;
      ConnectionH  dbcH[20], dbcH2 = 0;
      TransactionH trH1 = 0, trH2 = 0;
      Placement place1, place2, place3;
      Guid g1(getGuid("TNamed"));
      Guid g2(getGuid("test"));

      m_dbName2 += ".root";
      sc = unlink(m_dbName2.c_str())==0 ? DbStatus::Success : DbStatus::Error;
      checkError(sc,"Cannot delete file: "+m_dbName2);
      place2.setDb(m_dbName2);
      place2.setCont("C1");
      place2.setTechnology(m_type);
      place2.setShapeID(g2);

      for ( j = 0; j < 20; ++j )    {
        char txt4[128];
        ::sprintf(txt4,"%s_%d.root", m_dbName1.c_str(), int(j));
        std::string s = txt4;
        sc = unlink(txt4)==0 ? DbStatus::Success : DbStatus::Error;
        checkError(sc,"Cannot delete file: "+m_dbName1);
        sc = cache()->connect(s, poolDb::CREATE, dbcH[j]);
        checkError(sc,"Cannot connect to database "+s);
      }

      sc = cache()->connect(m_dbName2, poolDb::CREATE, dbcH2);
      checkError(sc,"Cannot connect to database "+m_dbName2);

      for ( i = 0; i < m_innerLoops; ++i )   {
        char txt1[256], txt2[256], txt3[256], txt4[256];
        ::sprintf(txt3,"Hello [test(%d)]", int(i));
        ::sprintf(txt4,"AnotherTNamed_%d", int(i));

        sc = cache()->startTransaction(dbcH2, trH2);
        checkError(sc,"Cannot start database transactions (2)");

        pool::Ref<test>   refObj(new test);

        refObj->m_string  = txt3;
        ::sprintf(refObj->m_text,"test_ntchar(%d)", int(i));
        refObj->m_char   = i % std::numeric_limits<char>::max();
        refObj->m_uchar  = i % std::numeric_limits<unsigned char>::max();
        refObj->m_short  = i % std::numeric_limits<short>::max();
        refObj->m_ushort = i % std::numeric_limits<unsigned short>::max();
        refObj->m_int    = i % std::numeric_limits<int>::max();
        refObj->m_uint   = i % std::numeric_limits<unsigned int>::max();
        refObj->m_long   = i % std::numeric_limits<long>::max();
        refObj->m_ulong  = i % std::numeric_limits<unsigned long>::max();
#ifdef __use_unkeyed
#ifdef __use_refs
        for ( j = 0; j < 20; ++j )    {
          ::sprintf(txt1,"Hello [TNamed(%d, %d)]", int(i), int(j));
          ::sprintf(txt2,"World [TNamed(%d, %d)]", int(i), int(j));
          ::sprintf(txt4,"%s_%d.root", m_dbName1.c_str(), int(j));
          place1.setDb(txt4);
          place1.setCont("C1");
          place1.setTechnology(ROOTKEY_StorageType.type());
          place1.setShapeID(g1);

          sc = cache()->startTransaction(dbcH[j], trH1);
          checkError(sc,"Cannot start database transactions (1)");
          pool::Ref<TNamed> refNamed(new TNamed(txt1, txt2));
          refObj->m_ref_vector.push_back(refNamed);
          sc = cache()->registerForWrite(trH1, place1, refNamed.ptr());
          if ( !sc.isSuccess() )   {
            checkError(sc,"Error cache.registerForWrite(trH1, place1, TNamed) [1]");
          }
          sc = cache()->commit(trH1);
          checkError(sc,"Error cache.commit(...) [1]");
        }
#endif
#endif
        sc = cache()->registerForWrite(trH2, place2, refObj.ptr());
        if ( !sc.isSuccess() )   {
          checkError(sc,"Error cache.registerForWrite(trH2, place2, test)   [2]");
        }
        sc = cache()->commit(trH2);
        checkError(sc,"Error cache.commit(...) [2]");
        sc = cache()->clear();
        checkError(sc,"Error cache.clear(...)");
      }
      std::cout << "Diconnect from database accesses..." << std::endl;
      for ( j = 0; j < 20; ++j )    {
        sc = cache()->disconnect(dbcH[j]);
        checkError(sc,"Error cache.disconnect(...) [1]");
      }
      sc = cache()->disconnect(dbcH2);
      checkError(sc,"Error cache.disconnect(...) [2]");
      return sc;
    }

    /// read objects from the persistent store
    DbStatus read()        {
      DbStatus sc;
      Guid g2(getGuid("test"));
      m_dbName2 += ".root";
      for ( size_t i = 0; i < m_innerLoops; ++i )   {
        for ( size_t j = 0; j < 20; ++j) {
          Token *tok2 = new Token();

          // Since this example does not use collections,
          // we do it here by faking the tokens to be 
          // returned by the collections
          // 
          tok2->setDb(m_dbName2);
          tok2->setCont("C1");
          tok2->setTechnology(m_type);
          tok2->setClassID(g2);
          tok2->oid().first  = 1;
          tok2->oid().second = i;

          if ( i % s_printoutMod == 0 ) {
            std::cout << "======================= New round....=====================" << std::endl;
          }

          // Read first object
          pool::Ref<test> refObj(cache(), tok2);
          if ( refObj && i%s_printoutMod == 0 )   {
#ifdef __use_primitives
            std::cout << " s:"      << refObj->m_string
                      << " char*:"  << refObj->m_text
                      << " char:"   << long(refObj->m_char)
                      << " short:"  << long(refObj->m_short)
                      << " int:"    << long(refObj->m_int)
                      << " long:"   << long(refObj->m_long)
                      << " uchar:"  << long(refObj->m_uchar)
                      << " ushort:" << long(refObj->m_ushort)
                      << " uint:"   << long(refObj->m_uint)
                      << " ulong:"  << long(refObj->m_ulong)
                      << std::endl;
#endif
            /*
             *      Test access to vector of references:
             */
#ifdef __use_unkeyed
#ifdef __use_refs
            TNamed* pNamed = refObj->m_ref_vector[j];
            if ( 0 == pNamed )  {
              pNamed = refObj->m_ref_vector[j];
            }
            std::cout << " refV:" << refObj->m_ref_vector.size() << "->";
            std::cout << "'" << pNamed->GetTitle() << "'";
            std::cout << std::endl;
#endif
#endif
          }
          cache()->clear();
        }
      }
      std::cout << "Clear cache..." << std::endl;
      sc = cache()->disconnect(0);
      checkError(sc,"cache.disconnect(0)");
      return sc;
    }
  };
}

pool::TestRunner<pool::FileAging> s_testFileAging("FileAging");

