#include "pool/Ref.h"
#include "pool/TestBase.h"
#include "pool/PerformanceObj.h"
#include "StorageSvc/DbType.h"
#include "StorageSvc/DbInstanceCount.h"
#include <ctime>

namespace pool  {

  class SizePerformance : public TestBase {
  public: 

    /// Standard Constructor
    SizePerformance() : TestBase(ROOTKEY_StorageType.type(), "perf_1.root", "perf_2.root") 
    {  m_innerLoops = 4000; }

    /// Standard Destructor
    virtual ~SizePerformance()    {    }

    /// write objects to the persistent store
    DbStatus write()       {
      DbStatus sc;
      ConnectionH  dbcH1 = 0;
      TransactionH trH1 = 0;

      sc = unlink(m_dbName1.c_str())==0 ? DbStatus::Success : DbStatus::Error;
      checkError(sc,"Cannot delete file: "+m_dbName1);

      sc = cache()->connect(m_dbName1, poolDb::RECREATE, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1);
      cache()->setDeleteObjects(false);

      size_t i, j, k, l;
      for ( l = 0; l < 5; ++l )   {
        char txt[64];
        std::vector<PerformanceObj*> objects;
        sprintf(txt,"cont_%d_%d", l, m_objSize);
        Placement place1(m_dbName1,txt, pool::Guid::null(), m_type);

        for ( k = 0; k < m_innerLoops; ++k )   {
          objects.push_back(new PerformanceObj(m_objSize));
        }
        clock_t start = clock();
        for ( j = 0; j < m_outerLoops; ++j )   {
          clock_t start2 = clock();
          sc = cache()->startTransaction(dbcH1, trH1);
          checkError(sc,"Cannot start database transactions (1)");
          clock_t start3 = clock();
          //std::cout << " [" << std::setw(4) << j << "] " << std::setw(12) << float(clock()-start2)/CLOCKS_PER_SEC << ": Register objects " << std::endl;
          for ( i = 0; i < m_innerLoops; ++i )   {
            pool::Ref<PerformanceObj > ref(objects[i]);
            sc = cache()->registerForWrite(trH1, place1, ref.ptr());
            if ( !sc.isSuccess() )   {
              checkError(sc,"Error cache.registerForWrite(trH1, place1, PerformanceObj) [1]");
            }
          }
          //std::cout << " [" << std::setw(4) << j << "] " << std::setw(12) << float(clock()-start2)/CLOCKS_PER_SEC << ": Commit transaction  " << std::endl;
          clock_t start4 = clock();
          sc = cache()->commit(trH1);
          checkError(sc,"Error cache.commit(...) [1]");
          //std::cout << " [" << std::setw(4) << j << "] " << std::setw(12) << float(clock()-start2)/CLOCKS_PER_SEC << ": Clear cache... " << std::endl;
          clock_t start5 = clock();
          sc = cache()->clear();
          checkError(sc,"Error cache.clear(...)");
          //std::cout << " [" << std::setw(4) << j << "] " << std::setw(12) << float(clock()-start2)/CLOCKS_PER_SEC << ": Done... "  << std::endl;
          clock_t start6 = clock();
          if ( j > m_outerLoops ) {
            std::cout << " [" << std::setw(4) << j << "] " 
              << std::setw(10) << float(start3-start2)/CLOCKS_PER_SEC 
              << std::setw(10) << float(start4-start3)/CLOCKS_PER_SEC 
              << std::setw(10) << float(start5-start4)/CLOCKS_PER_SEC 
              << std::setw(10) << float(start6-start5)/CLOCKS_PER_SEC 
              << std::setw(10) << float(start6-start2)/CLOCKS_PER_SEC 
              << " : Done (" << txt << ")" << std::endl;
          }
        }
        std::cout << " [" << std::setw(4) << j << "] " 
          << std::setw(10) << float(clock()-start)/CLOCKS_PER_SEC 
          << " " << std::setw(6) << m_objSize*sizeof(int) << " Bytes/Object " 
          << std::setw(7) << i*j << " Objects " 
          << std::setw(7) << j << " Transactions Container:" 
          << txt << std::endl;

        for ( k = 0; k < m_innerLoops; ++k )   {
          delete objects[k];
        }
        m_objSize = m_objSize*2;
      }
      dumpStreamerInfo("PerformanceObj");

      std::cout << "Diconnect from database accesses..." << std::endl;
      //sc = cache()->disconnect(dbcH1);
      checkError(sc,"Error cache.disconnect(...) [1]");
      return sc;
    }
    /// read objects from the persistent store
    DbStatus read()        {
      return poolDb::Error;
    }
  };
}

pool::TestRunner<pool::SizePerformance> s_testSizePerformance("SizePerformance");
