#include "TROOT.h"
#include "TFile.h"
#include "TTree.h"
#include "TBranch.h"
#include "TSystem.h"
#include <iostream>
#include "pool/TestBase.h"
#include "VirtualVector.h"

/*
rootcint VirtualVector_dict.cpp -c VirtualVector.h VirtualVector_LinkDef.h
g++ VirtualVector_dict.cpp -L${ROOTSYS}/lib -lCint -lCore -I${ROOTSYS}/include -ldl -fPIC -shared -o libEmb.so

rootcint VirtualVector_dict.cpp -c VirtualVector.h VirtualVector_LinkDef.h
cl.exe /c /DLL /MD /Z7 /GR /GX VirtualVector_dict.cpp /I"%ROOTSYS%/include"
link.exe VirtualVector_dict.obj /libpath:"%ROOTSYS%\lib" libCint.lib libCore.lib /DLL -out:libVirtualVector.dll
*/

namespace pool  {
  static int counter = 0;
  struct RootVirtualVector : public TestBase {

    /// Standard Constructor
    RootVirtualVector()    : TestBase(0, "", "")     {    }

    /// Standard Destructor
    virtual ~RootVirtualVector()               {    }
    
    /// write objects to the persistent store
    DbStatus write()       {
//      gSystem->Load("VirtualVector_dict.dll");
      TFile *f    = new TFile("RootVirtualVector.root", "RECREATE", "Root RootVirtualVector test");
      TTree *tree = new TTree("T","An example of a ROOT tree");
      VectorHolder* obj = new VectorHolder();
      TBranch *b = tree->Branch("B", "VectorHolder", &obj);
      for(size_t i=0; i < 10; ++i) {
        VirtualContained*    pv  = new VirtualContained();
        NonVirtualContained* pnv = new NonVirtualContained();
        obj->data.push_back(pv);
        obj->data.push_back(pnv);
        printf("%d\n",tree->Fill());
        obj->data.clear();
      }
      b->Print();
      b->Write();
      tree->Print();
      tree->Write();
      f->Print();
      f->Close();
      delete f;
      return 1;
    }
    /// write objects to the persistent store
    DbStatus read()       {
      TFile *f    = new TFile("RootVirtualVector.root");
      TTree *tree = (TTree*)f->Get("T");
      VectorHolder* obj = new VectorHolder();
      TBranch *branch  = tree->GetBranch("B");
      branch->SetAddress(&obj);
      Int_t nevent = Int_t(tree->GetEntries());
      Int_t nselected = 0;
      Int_t nb = 0;
      for (Int_t i=0;i<nevent;i++) { 
        printf("%d %d\n", tree->GetEvent(i), obj); 
        NonVirtualContained* pnv = dynamic_cast<NonVirtualContained*>(obj->data[1]);
        VirtualContained*    pv  = dynamic_cast<VirtualContained*>(obj->data[0]);
        printf("%f, %d\n", pv->data1, pnv->data2); 
        obj->data.clear();
      }
      return 1;
    }
  };
}

pool::TestRunner<pool::RootVirtualVector> s_testRootVirtualVector("RootVirtualVector");
