#ifndef COLLECTION_COLLECTIONSVC_H
#define COLLECTION_COLLECTIONSVC_H

#include "Collection/Collection.h"
#include "Collection/CollectionProxy.h"

#include "DataSvc/IDataSvc.h"
#include "FileCatalog/IFileCatalog.h"

namespace pool {

   class MetaDataEntry;
   //class IFileCatalog;
   
  /** 
      @class CollectionSvc CollectionSvc.h Collection/CollectionSvc.h
      
      @brief Collection Utility methods
      
      The provided factory methods allows to create and access collections that are registered in a catalog
      
  */
  
  class CollectionSvc {
  public:
    static void setDefaultCatalog(IFileCatalog* catalog);
    
    template <class T> static Collection<T> 
    createAndRegister( IDataSvc* dataSvc, 
		       const std::string& type,
		       const std::string& connection,
		       const std::string& name,
		       const std::string& logicalName = "",
		       MetaDataEntry* collectionMetaData = 0,
		       bool authorizeReplace = false,
		       IFileCatalog* catalog = 0 );
    
    static CollectionProxy
    createAndRegister( IDataSvc* dataSvc, 
		       const std::string& type,
		       const std::string& connection,
		       const std::string& name,
		       const std::string& logicalName = "",
		       MetaDataEntry* collectionMetaData = 0,
		       bool authorizeReplace = false,
		       IFileCatalog* catalog = 0 );
    
    
    
    template <class T> static Collection<T> 
    openWithPhysicalName( IDataSvc* dataSvc, 
			  const std::string& physicalName,
			  bool readOnly = true,
			  IFileCatalog* catalog = 0);
    
    template <class T> static Collection<T> 
    openWithLogicalName( IDataSvc* dataSvc, 
			 const std::string& logicalName,
			 bool readOnly = true,
			 IFileCatalog* catalog = 0);
    
    template <class T> static Collection<T> 
    openWithGuid( IDataSvc* dataSvc, 
		  const FileCatalog::FileID& id,
		  bool readOnly = true,
		  IFileCatalog* catalog = 0);
    

    
    static CollectionProxy 
    openWithPhysicalName( IDataSvc* dataSvc, 
			  const std::string& physicalName,
			  bool readOnly = true,
			  IFileCatalog* catalog = 0);
    
    static CollectionProxy 
    openWithLogicalName( IDataSvc* dataSvc, 
			 const std::string& logicalName,
			 bool readOnly = true,
			 IFileCatalog* catalog = 0);
    
    static CollectionProxy 
    openWithGuid( IDataSvc* dataSvc, 
		  const FileCatalog::FileID& id,
		  bool readOnly = true,
		  IFileCatalog* catalog = 0); 
    
    
    
    template <class T> static typename Collection<T>::Iterator 
    select( IDataSvc* dataSvc, 
	    const std::string& objectQuery = "",
	    const std::string& collectionQuery = "",
	    const std::string& objectIteratorOptions ="",
	    IFileCatalog* catalog = 0);
    
    static CollectionProxy::Iterator 
    select( IDataSvc* dataSvc, 
	    const std::string& objectQuery = "",
	    const std::string& collectionQuery = "",
	    const std::string& objectIteratorOptions ="",
	    IFileCatalog* catalog = 0);

  };

}

#include "CollectionSvc.inl"

#endif
  
