// 
#ifndef POOL_CACHEENTRY_H
#define POOL_CACHEENTRY_H 

#include "DataSvc/SharedAnyPtr.h"

namespace pool    {

  class Token;

  /** @class CacheEntry CacheEntry.h
      Unit of storage memory for the DataSvc cache. A SINGLE cache line is created 
      for both input object (defined by the object pointer) and for output object (defined by the
      storage token). Life of transient object read form DB depends directly on the life of the 
      related cache line.
      @author Giacomo Govi
      @date 10/2/2003
  */
  class CacheEntry {

    public:
  
    // default ctor
    CacheEntry();

    // default ctor
    CacheEntry(bool deleteObject);

    // Token ctor
    CacheEntry(const Token& aToken, bool deleteObject=true);

    //                                           bool deleteObject=true);
    CacheEntry(const SharedAnyPtr& aPtr, bool deleteObject=true);

    // copy ctor
    CacheEntry(const CacheEntry& aCacheEntry);

    // standard destructor
    virtual ~CacheEntry();

    // assignement operator
    CacheEntry& operator=(const CacheEntry& aCacheEntry);

    // clone object content
    virtual CacheEntry* cloneObjectContent();

    // attach to the entry a specified token
    bool setToken(const Token* aToken);

    // return the current Token pointer
    const Token* token() const;

    // attach to the cache line a specified object pointer/Class
    bool setObject(const SharedAnyPtr& aPtr);

    // return the object pointer
    const SharedAnyPtr& object() const;

    protected:

    // object pointer
    SharedAnyPtr m_objPtr;

    // Token pointer
    const Token* m_token;

    // method to destroy the cache line content
    void _destroyContent();

  };

}

#endif  // POOL_CACHEENTRY_H


