#ifndef POOL_DATASVCCONTEXT_H
#define POOL_DATASVCCONTEXT_H

#include "DataSvc/ObjectDeletePolicy.h"

namespace pool    {

  class IPersistencySvc;
  class ITokenValidator;
  class IDataTransform;
  class IFileCatalog;

/** @class DayaSvcContext DataSvcContext.h

    @author Giacomo Govi
    @date 09/07/2003
**/
  class DataSvcContext  {

    public:
    
    DataSvcContext(): m_tokenValidator(0),
                      m_fileCatalog(0),
                      m_dataTransform(0),
                      m_deletePolicy(ObjectDeletePolicy::defaultPolicy()){}

    virtual ~DataSvcContext(){}

    DataSvcContext(const DataSvcContext& aContext): m_tokenValidator(aContext.m_tokenValidator),
                                              m_fileCatalog(aContext.m_fileCatalog),
                                              m_dataTransform(aContext.m_dataTransform),
                                              m_deletePolicy(aContext.m_deletePolicy){}
      
    DataSvcContext& operator=(const DataSvcContext& aContext){
      m_tokenValidator = aContext.m_tokenValidator;
      m_fileCatalog = aContext.m_fileCatalog;
      m_dataTransform = aContext.m_dataTransform;
      m_deletePolicy = aContext.m_deletePolicy;
      return *this;
    }

    void setTokenValidator(ITokenValidator* aTokenValidator){
      m_tokenValidator = aTokenValidator;
    }

    void setFileCatalog(IFileCatalog* aFileCatalog){
      m_fileCatalog = aFileCatalog;
    }

    void setDataTransform(IDataTransform* aDataTransform){
      m_dataTransform = aDataTransform;
    }

    void setObjectDeletePolicy(const ObjectDeletePolicy& aPolicy){
      m_deletePolicy = aPolicy;
    }

    IFileCatalog* fileCatalog() const {
      return m_fileCatalog;
    }

    ITokenValidator* tokenValidator() const {
      return m_tokenValidator;
    }

    IDataTransform* dataTransform() const {
      return m_dataTransform;
    }

    const ObjectDeletePolicy& objectDeletePolicy() const {
      return m_deletePolicy;
    }
   
  private:

    ITokenValidator* m_tokenValidator;

    IFileCatalog* m_fileCatalog;

    IDataTransform* m_dataTransform;

    ObjectDeletePolicy m_deletePolicy;

  };   // class DataSvcContext
}      // namespace pool

#endif  // POOL_DATASVCCONTEXT_H
