#ifndef POOL_SHAREDANYPTR_H
#define POOL_SHAREDANYPTR_H

#include <typeinfo>
#include "DataSvc/AnyPtr.h"

#include <iostream>

namespace pool{

  class PtrHolder;
  class AnyType;
  class SharedAnyPtr {

  public: 
    
    SharedAnyPtr();

    explicit SharedAnyPtr(bool deletePolicy);

    template <class T> explicit SharedAnyPtr(T* p,bool deletePolicy = true);

    explicit SharedAnyPtr(const AnyPtr&, bool deletePolicy = true);
                     
    SharedAnyPtr(const SharedAnyPtr&);

    virtual ~SharedAnyPtr();

    SharedAnyPtr& operator=(const SharedAnyPtr&);

    SharedAnyPtr& copy(const SharedAnyPtr&);

    SharedAnyPtr& operator=(const AnyPtr&);

    template <class T> SharedAnyPtr& operator=(T*);

    template <class T> friend bool operator==(T* aPtr, 
                                              const SharedAnyPtr& anObj){
      return anObj.get()== (void*)aPtr;
    }
    template <class T> friend bool operator==(const SharedAnyPtr& anObj,
                                              T* aPtr){
      return anObj.get()== (void*)aPtr;
    }

    template <class T> friend bool operator!=(T* aPtr, 
                                              const SharedAnyPtr& anObj){
      return anObj.get()!= (void*)aPtr;
    }
    template <class T> friend bool operator!=(const SharedAnyPtr& anObj,
                                              T* aPtr){
      return anObj.get()!= (void*)aPtr;
    }

    bool operator==(const SharedAnyPtr& anObj) const;

    bool operator!=(const SharedAnyPtr& anObj) const;

    operator bool() const;

    const AnyPtr get() const; 

    const void* constPtr() const;

    void* ptr() const;

    //void* ptr(const std::type_info& asType) const;
    void* ptr(const AnyType& asType) const;

    //bool isOfType(const std::type_info& type) const;
    bool isOfType(const AnyType& type) const;

    void reset();

    unsigned int useCount();

    void forceDelete();

    bool deletePolicy();

    static void dumpRegister();

    static unsigned int numberOfInstances();

  private:

    PtrHolder* m_ptr;

    bool m_deletePolicy;

    PtrHolder* _register(const AnyPtr& aPtr);

  };

}

template <class T> pool::SharedAnyPtr::SharedAnyPtr(T* p,
                                                bool deletePolicy):
                                       m_ptr(),m_deletePolicy(deletePolicy){
  if(p){
    AnyPtr thePtr(p);
    m_ptr = _register(thePtr);
  }
}

template <class T> pool::SharedAnyPtr& 
                                     pool::SharedAnyPtr::operator=(T* pointer){
  SharedAnyPtr tmp(pointer,m_deletePolicy);
  operator=(tmp);
  return *this;
}

#endif
