#include "DataSvc/AnyPtr.h"
#include "Reflex/Object.h"

pool::AnyPtr::AnyPtr() : m_ptr(0),m_type(){}

pool::AnyPtr::AnyPtr(const void* aPointer, const std::type_info* aType):
                                              m_ptr(aPointer),m_type(){
  if(aType){
    m_type = *aType;
  }
}

#if ROOT_VERSION_CODE < ROOT_VERSION(5,19,0)
pool::AnyPtr::AnyPtr(const ROOT::Reflex::Object& anObj):
                                              m_ptr(anObj.Address()),m_type(anObj.TypeOf()){}
#else
pool::AnyPtr::AnyPtr(const Reflex::Object& anObj):
                                              m_ptr(anObj.Address()),m_type(anObj.TypeOf()){}
#endif

pool::AnyPtr::AnyPtr(
              const std::pair<void*,const TypeH>& objPair):
              m_ptr(objPair.first),m_type(objPair.second){}

pool::AnyPtr::AnyPtr(const AnyPtr &aPtr):
              m_ptr(aPtr.m_ptr),m_type(aPtr.m_type){}

pool::AnyPtr::~AnyPtr(){}

pool::AnyPtr& pool::AnyPtr::operator=(const AnyPtr& aPtr){
  m_ptr = aPtr.m_ptr;
  m_type = aPtr.m_type;
  return *this;
}

pool::AnyPtr::operator bool() const {
  return m_ptr != 0;
}

bool pool::AnyPtr::operator==(const AnyPtr& anObj) const {
  return m_ptr == anObj.m_ptr;
}

bool pool::AnyPtr::operator!=(const AnyPtr& anObj) const {
  return !(operator==(anObj));
}

const void* pool::AnyPtr::constPtr() const {
  return m_ptr;
} 

pool::AnyPtr::operator const void*() const {
  return constPtr();
}

void* pool::AnyPtr::ptr() const {
  return const_cast<void*>(m_ptr);
}

void* pool::AnyPtr::ptr(const AnyType& asType) const {
  return m_type.cast(asType,m_ptr);
}

bool pool::AnyPtr::isOfType(const AnyType& type) const {
  return m_type.hasBase(type);
}
     
const pool::AnyType& pool::AnyPtr::getType() const {
  return m_type;
}

 
const TypeH& pool::AnyPtr::getClass() const {
  return m_type.getClass();
}

void pool::AnyPtr::reset(){
  m_ptr = 0;
  m_type.reset();
}

void pool::AnyPtr::forceDelete(){
  void *thePtr = const_cast<void*>(m_ptr);
  AnyType theType = m_type;
  reset();
  theType.destroy(thePtr);
}
