#ifndef POOL_CACHE_H
#define POOL_CACHE_H

// Include files

#include "ICache.h"
#include "DataSvc/ObjectDeletePolicy.h"

namespace pool    {

  /** @class Cache Cache.h
      Implementation of the the abstract interface ICache.
      @author Giacomo Govi
      @date 25/07/2002
  **/
  class Cache : public ICache {
     
    public:

      // default constructor
      Cache();

      // ctor with a delete policy
      Cache(const ObjectDeletePolicy&);

      // virtual dtor
      virtual ~Cache();

      // retrieve the cache line correpondent to a given object pointer.
      CacheEntryImp* getEntry(const SharedAnyPtr& aPtr);

      // retrieve the cache line correpondent to a given token.
      CacheEntryImp* getEntry(const Token& aToken);

      // verify if a specified line belongs to the cache
      bool findEntry(const CacheEntryImp* aCacheEntryImp);

      // add the sepcified line to the cache
      bool addEntry(CacheEntryImp *line);

      // update the specified line with the specified object
      bool updateEntry(CacheEntryImp* aCacheEntryImp, const SharedAnyPtr& anObjPtr); 
		                         
      // update the specified line with the specified token
      bool updateEntry(CacheEntryImp* aCacheEntryImp, const Token& aToken  ); 

      // update the specified line resetting the object
      bool removeObject(CacheEntryImp* aCacheEntryImp);

      // update the specified line resetting and deleting the object on memory 
      bool deleteObject(CacheEntryImp* aCacheEntryImp);

      // update the specified line resetting the token
      bool removeToken(CacheEntryImp* aCacheEntryImp);

      // remove the specified line from the cache 
      bool removeEntry(CacheEntryImp* aCacheEntryImp);

      // iterators on the cache location list
      const_list_iterator list_begin();
      const_list_iterator list_end();

      // iterators on the object pointer table
      const_ptr_iterator ptr_iter_begin();
      const_ptr_iterator ptr_iter_end();

      // iterators on the token table
      const_token_iterator token_iter_begin();
      const_token_iterator token_iter_end();

     // clear the cache tables, without to delete lines and contents
     void clearTable();

     // return the size of the cache lines list.
     unsigned int cacheSize();

    private:

      // private instance of the CacheTable object
      CacheTable* m_table; 

      // delete policy
      ObjectDeletePolicy m_deletePolicy;      

  };   // class CacheMgr
}      // namespace pool

#endif  // POOL_CACHE_H
