#include "CacheEntryImp.h" 

// default ctor
pool::CacheEntryImp::CacheEntryImp(): 
          RefCounter(),CacheEntry(),
          marked_write(false),                          
	  marked_update(false), 
          marked_delete(false) {
}
   
pool::CacheEntryImp::CacheEntryImp(bool deleteObject): 
          RefCounter(),CacheEntry(deleteObject),
          marked_write(false),
          marked_update(false), 
          marked_delete(false) {
}   

// Token ctor
pool::CacheEntryImp::CacheEntryImp(const Token& aToken, bool deleteObject): 
          RefCounter(),CacheEntry(aToken,deleteObject),
          marked_write(false),
          marked_update(false),
          marked_delete(false){ }                         

pool::CacheEntryImp::CacheEntryImp(const SharedAnyPtr& aPtr, bool deleteObject): 
            RefCounter(),CacheEntry(aPtr,deleteObject),
            marked_write(false),
            marked_update(false),
            marked_delete(false){ }


pool::CacheEntryImp::CacheEntryImp(const CacheEntryImp& aCacheEntryImp):
           RefCounter(aCacheEntryImp),
           CacheEntry(aCacheEntryImp),
           marked_write(aCacheEntryImp.marked_write),
           marked_update(aCacheEntryImp.marked_update),
           marked_delete(aCacheEntryImp.marked_delete){
}

// standard destructor
pool::CacheEntryImp::~CacheEntryImp() {}

pool::CacheEntryImp& pool::CacheEntryImp::operator=(const CacheEntryImp& aCacheEntryImp){
  RefCounter::operator=(aCacheEntryImp);
  marked_write = aCacheEntryImp.marked_write;  
  marked_update = aCacheEntryImp.marked_update;                          
  marked_delete = aCacheEntryImp.marked_delete;                          
  return *this;
}

pool::CacheEntry* pool::CacheEntryImp::cloneObjectContent(){
  CacheEntry* loc = new CacheEntryImp(m_objPtr,m_objPtr.deletePolicy());
  return loc;
}

// reset the fields describing the object
bool pool::CacheEntryImp::resetObject(){
  bool ret = false;
  marked_write  = false;          
  marked_update = false;          
  marked_delete = false;
  m_objPtr.reset();
  return ret;
}

// reset the fields describing the object
bool pool::CacheEntryImp::deleteObject(){
  bool ret = false;
  marked_write  = false;          
  marked_update = false;          
  marked_delete = false;
  m_objPtr.forceDelete();
  return ret;
}

// reset all fields of the cache line but the reference counter
void pool::CacheEntryImp::reset(){
  _destroyContent();  
  marked_write = false;  
  marked_update = false;  
  marked_delete = false;  
}

// set the cache line content to be marked for write. After the commit the object pointer
// will be reset
void pool::CacheEntryImp::setMarkWrite(bool set_flag){
  marked_write = set_flag;
}

// verifies if the current object has been marked for write
bool pool::CacheEntryImp::isMarkedWrite() const {
  return marked_write;
}

// set the cache line content to be marked for update. After the commit the object pointer
// will be reset
void pool::CacheEntryImp::setMarkUpdate(bool set_flag){
  marked_update = set_flag;
}

// verifies if the current object has been marked for update
bool pool::CacheEntryImp::isMarkedUpdate() const {
  return marked_update;
}

// set the cache line content to be marked for delete. After the commit the object pointer
// will be reset
void pool::CacheEntryImp::setMarkDelete(bool set_flag){
  marked_delete = set_flag;
}

// verifies if the current object has been marked for delete
bool pool::CacheEntryImp::isMarkedDelete() const {
  return marked_delete;
}

void pool::CacheEntryImp::unMark() {
  marked_write = false;
  marked_update = false;
  marked_delete = false;
}

bool pool::CacheEntryImp::isMarked() const {
  return marked_write || marked_update || marked_delete;
}
