#ifndef POOL_REFERENCEHANDLER_H
#define POOL_REFERENCEHANDLER_H

// Framework include files
#include "StorageSvc/DbStatus.h"
#include "POOLCore/Token.h"
#include "StorageSvc/DbDefaultDataHandler.h"

/*
 *    POOL namespace
 */
namespace pool    {

  // Forward declarations
  class ICacheSvc;

  /** @class ReferenceHandler ReferenceHandler.h
    * 
    * Exampel class to show specialized reference handling
    * depending on the implementation of the data cache.
    *
    * @author  G.Govi
    * @version 1.0
    * @date    10/02/2003
    */
  class ReferenceHandler : public DbDefaultDataHandler {
    /// reference to data cache
    ICacheSvc* m_cache;
  public:

    /// Standard Constructor: Tkes data cache as argument.
    ReferenceHandler(ICacheSvc* cache);

    /// Standard Destructor
    virtual ~ReferenceHandler();

    /// Internal call on saving references.
    /** Convert transient reference to persistent OID.
      * @param pRef           [IN] Pointer to reference object to be saved
      * @param pOid           [IN] Pointer to OID data for this reference
      *                            The oid is all, what is stored in the
      *                            persistent world.
      *
      * @return Status code indicating success or failure
      */
    virtual pool::DbStatus saveReference(Reference* pRef, Token::OID_t* data);

    /// Internal call on loading references.
    /** Convert persistent OID into a transient reference.
      * @param pRef           [IN] Pointer to reference object to be loaded
      * @param pOid           [IN] Pointer to OID data for this reference
      *                            The oid is all, what is stored in the
      *                            persistent world.
      *
      * @return Status code indicating success or failure
      */
    virtual pool::DbStatus loadReference(Reference* pRef, Token::OID_t* data);
  };
}
#endif // POOL_REFERENCEHANDLER_H
