// $Header: /cvs/PF/pool/DataSvc/tests/Common/DummyPersistencySvc.h,v 1.22 2008/03/06 17:58:58 govi Exp $
#ifndef POOL_DUMMYPERSSVC_H
#define POOL_DUMMYPERSSVC_H

/**
    POOL namespace
*/

#include "TestDataObject.h"
#include "POOLCore/Token.h"
#include <utility>
#include "PersistencySvc/IPersistencySvc.h"
#include "PersistencySvc/ITransaction.h"
#include "PersistencySvc/IConfiguration.h"
#include "StorageSvc/DatabaseConnection.h"
#include "PersistencySvc/ISession.h"
#include "PersistencySvc/DatabaseConnectionPolicy.h"

// Import declarations
namespace reflex {  class Type;    }

namespace pool {

class DummyTransaction : public pool::ITransaction {
public:
  DummyTransaction() {}
  virtual ~DummyTransaction() {}

  /// Starts a new transaction. Returns the success of the operation
  bool start(Type){
    return true;
  }

  /// Commits the transaction. If unsuccessfull the transaction is aborted.
  bool commit(){
    return true;
  }

  /// Commits and holds the transaction.
  bool commitAndHold(){
    return true;
  }

  /// Rolls back the transaction
  void rollback(){}

    /// Checks if the transaction is active
  bool isActive() const {
    return true;
  }

  /// Reverts to update mode
  void update(){}

  /// Returns the transaction type
  Type type() const {
    return ITransaction::UNDEFINED;
  }

};

 class DummySession: public pool::ISession {

 public:
   DummySession(){
     m_policy = new DatabaseConnectionPolicy;
     m_transaction = new DummyTransaction;
   }

   virtual ~DummySession(){
     delete m_policy;
     delete m_transaction;
   }

    /// Sets the default policy when databases are opened/connected
   void setDefaultConnectionPolicy( const DatabaseConnectionPolicy& ){}

    /// Retrieves the default connection policy
   const DatabaseConnectionPolicy& defaultConnectionPolicy() const {
     return *m_policy;     
   }

    //  Explicitly disconnects all the databases.
    void disconnectAll(){}

    /// Returns the transaction object
    ITransaction& transaction(){
      return *m_transaction;
    }
    const ITransaction& transaction() const {
      return *m_transaction;
    }

    /// Returns a vector with the file identifiers of the presently open databases.
    std::vector< std::string > connectedDatabases() const {
      std::vector< std::string > vec;
      return vec;
    }

    /// Returns a pointer to a database object. The user acquires ownership of that object.
    IDatabase* databaseHandle( const std::string&,
				       DatabaseSpecification::NameType ){
      return 0;
    }

    IFileCatalog& fileCatalog(){
      return *catalog;
    }

    IShapeTransformationRegistry& shapeTransformationRegistry(){
      return *registry;
    }

    const ITechnologySpecificAttributes& technologySpecificAttributes(long) const {
      return *attributes;
    }
 
    ITechnologySpecificAttributes& technologySpecificAttributes(long) {
      return *attributes;
    }

 private:
    
    DatabaseConnectionPolicy* m_policy;

    DummyTransaction* m_transaction;

    IFileCatalog* catalog;

    IShapeTransformationRegistry* registry;

    ITechnologySpecificAttributes* attributes;

 };

 class DummyPersistencySvcConfiguration : virtual public IConfiguration {
    public:
      /// Constructor. Creates a dummy token validator and a default data transform
   DummyPersistencySvcConfiguration(){}
   
      /// Destructor
      virtual ~DummyPersistencySvcConfiguration(){}
      
      /** Sets the attached file catalog
       *
       *  @param catalog   [IN] Reference to the file catalog
       */
      void setFileCatalog( IFileCatalog& catalog ){
        m_fileCatalog = &catalog;
      }
      
      /// Retrieves the file catalog in use.
      IFileCatalog& fileCatalog(){
        return *m_fileCatalog;
      }
      
      const IFileCatalog& fileCatalog() const{
          return *m_fileCatalog;    
      }
      
      
      /** Sets the attached object token validator
       *
       *  @param validator [IN] Reference to the external validator
       */
      void setTokenValidator( ITokenValidator& validator ){
        m_tokenValidator = &validator;
      }
      
      
      /// Retrieves the token valitator in use
      ITokenValidator& tokenValidator(){
        return *m_tokenValidator;
      }
      
      /** Sets the attached object data transform
       *
       *  @param transform [IN] Reference to the external data transform
       */
      void setDataTransform( IDataTransform& transform ){
        m_dataTransform = &transform;
      }
      
      /// Returns a reference to the IDataTransform object.
      IDataTransform& dataTransform(){
        return *m_dataTransform;
      }

      /// Sets the database connection policy
      void setDatabaseConnectionPolicy( const DatabaseConnectionPolicy& policy ){
        m_policy = policy;
      }
      
      /// Returns the database connection policy.
      const DatabaseConnectionPolicy& databaseConnectionPolicy() const{
        return m_policy;
      }

    private:
      /// The file catalog
      IFileCatalog* m_fileCatalog;
      /// The token validator
      ITokenValidator* m_tokenValidator;
      /// The data transform
      IDataTransform* m_dataTransform;
      /// The default database connection policy
      DatabaseConnectionPolicy m_policy;
 };

class DummyPersistencySvc: public IPersistencySvc {

  public:

    DummyPersistencySvc(){
      m_session = new DummySession;
      m_configuration = new DummyPersistencySvcConfiguration;
    }

    DummyPersistencySvc(IFileCatalog& aCatalog){
      m_catalog = &aCatalog;
      m_session  = new DummySession;
      m_configuration = new DummyPersistencySvcConfiguration;
    }

    virtual ~DummyPersistencySvc(){
      delete m_session;
      delete m_configuration;
   }

    /// Returns the file catalog in use
    IFileCatalog& fileCatalog(){
      return *m_catalog;
    }

    /// Set the attached file catalog
    void setFileCatalog(IFileCatalog& pCatalog){
      m_catalog = &pCatalog;
    }

    /// Returns the token validator in use
    ITokenValidator& tokenValidator(){
      return *m_tokenValidator;
    }

    /// Set the attached object token validator
    void setTokenValidator(ITokenValidator& pValidator){
      m_tokenValidator = &pValidator;
    }

    /// Returns a reference to the IDataTransform object.
    IDataTransform& dataTransform(){
      return *m_dataTransform;
    }

    /// Sets the attached object data transform
    void setDataTransform( IDataTransform& dataTransform ){
      m_dataTransform = &dataTransform;
    }

    IConfiguration& configuration(){
      return *m_configuration;
    }
    

    // Retrieves an object from persistent store and return with type information
    std::pair< void*, ROOT::Reflex::Type > readObject( const Token& objToken,
									DataHandler* ){
      std::pair<unsigned long,unsigned long> id = objToken.oid();
      TestDataObject* theObj = new TestDataObject(id.second);
      ROOT::Reflex::Type clazz;
      std::pair<void*, ROOT::Reflex::Type> ret(theObj,clazz);
      return ret;

    }

    std::pair<void*, ROOT::Reflex::Type> readObject(const pool::Guid&, const pool::Token& token, pool::DataHandler*){
      std::pair<unsigned long,unsigned long> id = token.oid();
      TestDataObject* theObj = new TestDataObject(id.second);
      ROOT::Reflex::Type clazz;
      std::pair<void*, ROOT::Reflex::Type> ret(theObj,clazz);
      return ret;
    }
    
    std::pair<void*, ROOT::Reflex::Type> readObject(const ROOT::Reflex::Type&, const pool::Token& token, pool::DataHandler*){
      std::pair<unsigned long,unsigned long> id = token.oid();
      TestDataObject* theObj = new TestDataObject(id.second);
      ROOT::Reflex::Type clazz;
      std::pair<void*, ROOT::Reflex::Type> ret(theObj,clazz);
      return ret;
    }

    //  registerForWrite registers an object for writing to the persistent medium
    Token* registerForWrite( const Placement&,const void* object,
			     const ROOT::Reflex::Type&,DataHandler* ){
      const TestDataObject* theObj = 
                            static_cast<const TestDataObject*>(object);
      Token* tok = 0;
      if(theObj){
        long id = theObj->getOId();
        tok = new FakeToken(id);
      }
      return tok;
    }

    
    //  updateObject updates an object which is already written in the persistent medium.
    bool updateObject( const void*,
		       const ROOT::Reflex::Type&,
		       const Token&,
		       DataHandler* ){
      return true;
    }

    // Deletes an object from the persistent store
    bool destroyObject( const Token&){
      return true;
    }

    /// Returns the underlying global session
    ISession& session(){
      return *m_session;
    }

    /// Registers a transaction observer
    bool registerObserver( ITransactionObserver&){
      return true;
    }

    /// Deregisters a transaction observer
    bool deregisterObserver( ITransactionObserver&){
      return true;
    }

  private: 
    IFileCatalog* m_catalog;
    ITokenValidator* m_tokenValidator;
    IDataTransform* m_dataTransform;
    ISession* m_session;
    DummyPersistencySvcConfiguration* m_configuration;

  }; // class DummyPersistencySvc
} // namespace pool

#endif  // POOL_DUMMYPERSSVC_H
