#ifndef OBJECTRELATIONALACCESS_OBJECTRELATIONALDEFAULTMAPPINGRULES_H
#define OBJECTRELATIONALACCESS_OBJECTRELATIONALDEFAULTMAPPINGRULES_H

#include <string>
#include <locale>
#include <algorithm>
#include "RVersion.h"

#if ROOT_VERSION_CODE < ROOT_VERSION(5,19,0)
namespace ROOT{
  namespace Reflex{
    class Type;
  }
}
using namespace ROOT;
#else
namespace Reflex{
  class Type;
}
#endif 

namespace pool {

  class ObjectRelationalDefaultMappingRules {
    public:

      static const size_t MaxTableNameLength  = 20;
      static const size_t MaxColumnNameLength = 30;
      
      static const size_t ClassNameLengthForSchema  = 7;
    
    public:

      /// Returns the name of the index for the identity of an object mapped to a table
      static std::string indexNameForIdentity( const std::string& tableName );
      /// Returns the name of the foreign key to the identity of the parrent table
      static std::string fkNameForIdentity( const std::string& tableName );

      static std::string newMappingVersionForClass( const std::string& className, int iteration );

      static std::string classVersionPropertyNameInDictionary();

      static std::string defaultClassVersion(const std::string& className);

    public:
      /// Returns the default name for a table given a class name
      static std::string fullNameForSchema( const std::string& parentVariableName, const std::string& variableName );
      static std::string tableNameForClass( const std::string& className, size_t maxLength=MaxTableNameLength );
      static std::string tableNameForVariable( const std::string& variableName,const std::string& mainTableName );
      static std::string columnNameForId();
      static std::string columnNameForRefColumn();
      static std::string columnNameForPosition();
      static std::string columnNameForVariable( const std::string& variableName, const std::string& scope, bool forData=true );
      static std::string columnNameForOID_1( const std::string& variableName, const std::string& scope );
      static std::string columnNameForOID_2( const std::string& variableName, const std::string& scope );

      static std::string formatForSchema( const std::string& variableName, size_t maxLength);
      static std::string newNameForSchemaObject( const std::string& initialName, unsigned int index, size_t maxLength);
      /// Forms the scoped variable names for all of the fields
      static std::string scopedVariableName( const std::string& variableName, const std::string& scope );
      /// Forms the scoped variable names for the schema objects
      static std::string scopedVariableForSchemaObjects( const std::string& variableName, const std::string& scope );
      /// Forms the variable names for the c-array variable
      static std::string variableNameForArrayIndex( const std::string& arrayVariable, unsigned int index );
      /// Forms the variable names for the c-array columns
      static std::string variableNameForArrayColumn( unsigned int arrayIndex );
      static std::string variableNameForArrayColumn( const std::string& arrayVariable, unsigned int arrayIndex );

      static std::string variableNameForArrayColumn( const Reflex::Type& array );
      
      static std::string referenceColumnKey( const std::string& tableName, const std::string& columnName );
      
      static std::string variableNameForContainerValue();
      static std::string variableNameForContainerKey();
      
    private:

      static std::string shortName( const std::string& nameCut, size_t maxLenght);
      static std::string nameForSchema( const std::string& variableName );

      struct ToUpper {
          ToUpper(const std::locale& l):loc(l) {}
          char operator() (char c) const  { return std::toupper(c,loc); }
          std::locale const& loc;
      };
  };
}

#endif

