#ifndef OBJECTRELATIONALACCESS_OBJECTRELATIONALMAPPING_H
#define OBJECTRELATIONALACCESS_OBJECTRELATIONALMAPPING_H

#include "ObjectRelationalMappingElement.h"

namespace pool {

  /**
   * @class ObjectRelationalMapping ObjectRelationalMapping.h ObjectRelationalAccess/ObjectRelationalMapping.h
   * 
   * The structure holding an object/relational mapping.
   */
  
  class ObjectRelationalMapping
  {
    
    /** Public methods: */
  public:
    /// Constructor
    explicit ObjectRelationalMapping( const std::string& version );

    /// Destructor
    ~ObjectRelationalMapping() {}

    void setVersion( const std::string& version);
    
    /**
     * Returns the version of the mapping
     */
    const std::string& version() const;

    /**
     * Appends the element to the structure
     * @param className The class name of the new element
     * @param tableName The table name of the new element
     * @param elementType The type code of the new element
     */
    ObjectRelationalMappingElement& setElement( const std::string& className,
                                                const std::string& tableName );

    /**
     * Returns the main mapping element
     */
    const ObjectRelationalMappingElement& element() const;
    /**
     * Returns the main mapping element
     */
    ObjectRelationalMappingElement& element();
    
    /**
     * Returns the main class 
     */
    const std::string& className() const;

    /**
     * Appends the a new dependent element to the structure
     * @param className The class name of the new element
     * @param tableName The table name of the new element
     * @param elementType The type code of the new element
     */
    ObjectRelationalMappingElement& appendDependentElement( const std::string& className,
                                                            const std::string& tableName );

    /**
     * Returns the dependent elements map
     */
    const std::map< std::string, pool::ObjectRelationalMappingElement >& dependentElements() const;
    /**
     * Returns the dependent elements map
     */
    std::map< std::string, pool::ObjectRelationalMappingElement >& dependentElements();
    
    /**
     * Returns the version of the mapping
     */
    std::set<std::string> involvedTables() const;
    
    /// the number of dependent elements
    size_t numberOfDependentElements() const;

    /// replace present data with the provided source
    void override(const ObjectRelationalMapping& source);

  private:
    /**
     * The mapping version
     */
    std::string m_version;

    /**
     * The main class
     */
    std::string m_className;

    /**
     * The main tree
     */
    ObjectRelationalMappingElement m_element;

    /**
     * The top-level mapping elements
     */
    std::map< std::string, ObjectRelationalMappingElement > m_dependentElements;
  };
}

#include "ObjectRelationalMapping.inl"

#endif // OBJECTRELATIONALACCESS_OBJECTRELATIONALMAPPING_H
