#ifndef OBJECTRELATIONALACCESS_OBJECTRELATIONALMAPPINGUTILITIES_H
#define OBJECTRELATIONALACCESS_OBJECTRELATIONALMAPPINGUTILITIES_H

#include "ObjectRelationalAccess/ObjectRelationalUtilities.h"

namespace coral {
  class ISessionProxy;
}

namespace pool {

  class MappingBuilder;
  class ObjectRelationalMapping;
  class ObjectRelationalDatabaseDescriptor;
  
  class ObjectRelationalMappingUtilities: public ObjectRelationalUtilities  {

    public:
    ObjectRelationalMappingUtilities();

    ObjectRelationalMappingUtilities(coral::ISessionProxy* session);

    virtual ~ObjectRelationalMappingUtilities();

    void buildAndMaterializeMapping(const std::string& fileName, const std::string& outFile, bool evolve, bool dryRun=false);

    void buildAndMaterializeMapping(const ObjectRelationalDatabaseDescriptor& descriptor, const std::string& outFile, bool evolve, bool dryRun=false);

    void buildAndMaterializeMappingFromBuffer(const char* buffer, bool evolve, bool dryRun=false);

    void retrieveMapping(const std::string& version,const std::string& fileName);

    void retrieveMapping(const std::string& fileName, bool fromOldSchema=false);

    std::string getContainerMapping(const std::string& containerName, std::string classVersion=std::string(""));

    std::string getAllContainerMappings(const std::string& containerName);

    void retrieveContainerMapping(const std::string& containerName, const std::string& fileName, std::string classVersion=std::string("") );

    void retrieveAllContainerMappings(const std::string& containerName, const std::string& fileName );

    std::map<std::pair<std::string,std::string>, std::string > mappingForClassVersions();
    
    std::map<std::string,std::string> listMappings();

    std::map<std::string, std::string> containerTableMap();

    void selectMapping(const std::string& version, std::map<std::pair<std::string,std::string>,std::pair<std::string,std::string> >& mappingStruct);

    bool existsMapping(const std::string& version);

    bool removeMapping(const std::string& version, bool removeTables=false, bool dryRun=false);

    bool upgradeMappingSchema();

    bool loadMappingInformation( const std::string& containerName, const std::string& classVersion, bool all=false);

    bool storeMappingInformation( bool materialize );

    ObjectRelationalDatabaseDescriptor* loadDescriptor(const std::string fileName);

    ObjectRelationalDatabaseDescriptor* loadDescriptorFromBuffer(const char* buffer);

    static void prettyPrint(const std::map<std::string,std::string>& data, const std::string& firstColumn,const std::string& secondColumn, std::ostream& out );

    static void prettyPrint(const std::map<std::pair<std::string,std::string>,std::string>& data,
                            const std::string& firstColumn,const std::string& secondColumn, const std::string& thirdColumn, std::ostream& out );

    static void prettyPrint(const std::map<std::pair<std::string,std::string>,std::pair<std::string,std::string> >& data,
                            const std::string& firstColumn,const std::string& secondColumn,
                            const std::string& thirdColumn,const std::string& fourthColumn, std::ostream& out );

    static std::string keyOfMappingList();

    static std::string valueOfMappingList();

    static std::string keyFirstOfClassVersionList();
    
    static std::string keySecondOfClassVersionList();

    static std::string valueOfClassVersionList();

    static std::string keyOfContainerTableMap();

    static std::string valueOfContainerTableMap();

    private:

    void cleanBuffers();

    private:
    
    std::map<std::string,ObjectRelationalMapping*>* m_mappingBuffer;

    std::map<std::string,std::pair<std::string,std::string> >* m_versionBuffer;
    
  };

}

#endif
