#include "ObjectRelationalAccess/ObjectRelationalClassUtils.h"
#include <sstream>

bool pool::ObjectRelationalClassUtils::isTypeContainer(const Reflex::Type& typ){
  Reflex::TypeTemplate templ = typ.TemplateFamily();
  if (! templ) {
    return false;
  } else {
    std::string contName = templ.Name(Reflex::SCOPED|Reflex::FINAL); 
    if(  contName == "std::vector"              ||
         contName == "std::list"                ||
         contName == "std::deque"               ||
         contName == "std::stack"               ||
         contName == "std::set"                 ||
         contName == "std::multiset"            ||
         contName == "__gnu_cxx::hash_set"      ||
         contName == "__gnu_cxx::hash_multiset" ||
         contName == "std::map"                 ||
         contName == "std::multimap"            ||
         contName == "__gnu_cxx::hash_map"      ||
         contName == "__gnu_cxx::hash_multimap" ||
         contName == "pool::PVector" ){
       return true;
    }
  }
  return false;
}

bool pool::ObjectRelationalClassUtils::isTypeKeyedContainer(const Reflex::Type& typ){
  Reflex::TypeTemplate tt = typ.TemplateFamily();
  if (! tt) {
    return false;
  } else {
    std::string contName = tt.Name(Reflex::SCOPED|Reflex::FINAL); 
    if(  contName == "std::map"                 ||
         contName == "std::multimap"            ||
         contName == "std::set"                 ||
         contName == "std::multiset"            ||
         contName == "__gnu_cxx::hash_set"      ||
         contName == "__gnu_cxx::hash_multiset" ||
         contName == "__gnu_cxx::hash_map"      ||
         contName == "__gnu_cxx::hash_multimap" ){
      return true;
    }
  }  
  return false;
}

bool pool::ObjectRelationalClassUtils::isTypeNonKeyedContainer(const Reflex::Type& typ){
  Reflex::TypeTemplate templ = typ.TemplateFamily();
  if (! templ) {
    return false;
  } else {
    std::string contName = templ.Name(Reflex::SCOPED|Reflex::FINAL); 
    if(  contName == "std::vector"              ||
         contName == "std::list"                ||
         contName == "std::deque"               ||
         contName == "std::stack"               ||
         contName == "pool::PVector" ){
       return true;
    }
  }
  return false;
}
 
bool pool::ObjectRelationalClassUtils::isTypeAssociativeContainer(const Reflex::Type& typ){
  Reflex::TypeTemplate tt = typ.TemplateFamily();
  if (! tt) {
    return false;
  } else {
    std::string contName = tt.Name(Reflex::SCOPED|Reflex::FINAL); 
    if(  contName == "std::map"            ||
         contName == "std::multimap"       ||
         contName == "__gnu_cxx::hash_map" ||
         contName == "__gnu_cxx::hash_multimap" ){
      return true;
    }
  }  
  return false;
}

bool pool::ObjectRelationalClassUtils::isTypeNonAssociativeContainer(const Reflex::Type& typ){
  Reflex::TypeTemplate tt = typ.TemplateFamily();
  if (! tt) {
    return false;
  } else {
    std::string contName = tt.Name(Reflex::SCOPED|Reflex::FINAL); 
    if(  contName == "std::vector"              ||
         contName == "std::list"                ||
         contName == "std::deque"               ||
         contName == "std::stack"               ||
         contName == "std::set"                 ||
         contName == "std::multiset"            ||
         contName == "__gnu_cxx::hash_set"      ||
         contName == "__gnu_cxx::hash_multiset" ||
         contName == "pool::PVector"){
       return true;
    }
  }
  return false;
}


bool pool::ObjectRelationalClassUtils::isTypePoolPointer( const Reflex::Type& typ){
  Reflex::TypeTemplate templ = typ.TemplateFamily();
  if (! templ) {
    return false;
  } else {
    std::string contName = templ.Name(Reflex::SCOPED|Reflex::FINAL); 
    if(  contName == "pool::Ptr" ){
       return true;
    }
  }
  return false;  
}

bool pool::ObjectRelationalClassUtils::isTypePoolPolymorphicPointer( const Reflex::Type& typ){
  Reflex::TypeTemplate templ = typ.TemplateFamily();
  if (! templ) {
    return false;
  } else {
    std::string contName = templ.Name(Reflex::SCOPED|Reflex::FINAL); 
    if(  contName == "pool::PolyPtr" ){
       return true;
    }
  }
  return false;  
}

bool pool::ObjectRelationalClassUtils::isTypePVector( const Reflex::Type& typ){
  Reflex::TypeTemplate templ = typ.TemplateFamily();
  if (! templ) {
    return false;
  } else {
    std::string contName = templ.Name(Reflex::SCOPED|Reflex::FINAL); 
    if(  contName == "pool::PVector" ){
       return true;
    }
  }
  return false;  
}

Reflex::Type pool::ObjectRelationalClassUtils::containerValueType(const Reflex::Type& typ){
  Reflex::Type valueType;
  // find the iterator return type as the member value_type of the containers  
  size_t subTypeSize = typ.SubTypeSize();
  size_t i=0;
  while(i<subTypeSize){
    Reflex::Type sti = typ.SubTypeAt(i);    
    if(sti.Name()=="value_type") {
      valueType = sti;
      break;
    }    
    i++;
  }
  return valueType;
}

Reflex::Type pool::ObjectRelationalClassUtils::containerKeyType(const Reflex::Type& typ){
  Reflex::Type keyType;
  // find the iterator return type as the member value_type of the containers  
  size_t subTypeSize = typ.SubTypeSize();
  size_t i=0;
  while(i<subTypeSize){
    Reflex::Type sti = typ.SubTypeAt(i);    
    if(sti.Name()=="key_type") {
      keyType = sti;
      break;
    }    
    i++;
  }
  return keyType;
}

Reflex::Type pool::ObjectRelationalClassUtils::containerDataType(const Reflex::Type& typ){
  Reflex::Type dataType;
  // find the iterator return type as the member value_type of the containers  
  size_t subTypeSize = typ.SubTypeSize();
  size_t i=0;
  while(i<subTypeSize){
    Reflex::Type sti = typ.SubTypeAt(i);    
    if(sti.Name()=="mapped_type") {
      dataType = sti;
      break;
    }    
    i++;
  }
  return dataType;
}

Reflex::Type pool::ObjectRelationalClassUtils::resolvedType(const Reflex::Type& typ){
  Reflex::Type resolvedType = typ;
  while(resolvedType.IsTypedef()){
    resolvedType = resolvedType.ToType();
  }
  return resolvedType;
}
