#include "ObjectRelationalAccess/ObjectRelationalDatabaseDescriptor.h"
#include "ObjectRelationalAccess/ObjectRelationalMappingDescriptor.h"

pool::ObjectRelationalClassDescriptor::ObjectRelationalClassDescriptor():m_className(""),m_mappingVersion(""),m_mapping(0){}

pool::ObjectRelationalClassDescriptor::ObjectRelationalClassDescriptor(const std::string className,
                                                                       const std::string mappingVersions):m_className(className),
                                                                                                          m_mappingVersion(mappingVersions),m_mapping(0){}

pool::ObjectRelationalClassDescriptor::~ObjectRelationalClassDescriptor(){
  if(m_mapping) delete m_mapping;
}

const std::string& pool::ObjectRelationalClassDescriptor::className() const {
  return m_className;
}

const std::string& pool::ObjectRelationalClassDescriptor::mappingVersion() const {
  return m_mappingVersion;
}

const pool::ObjectRelationalMappingDescriptor* pool::ObjectRelationalClassDescriptor::mapping() const {
  return m_mapping;
}

pool::ObjectRelationalMappingDescriptor* pool::ObjectRelationalClassDescriptor::newMapping(){
  if(m_mapping) delete m_mapping;
  m_mapping = new ObjectRelationalMappingDescriptor;
  return m_mapping;
}

pool::ObjectRelationalContainerDescriptor::ObjectRelationalContainerDescriptor():m_name(""),m_type(""),m_classStores(),m_recreate(true){}

pool::ObjectRelationalContainerDescriptor::ObjectRelationalContainerDescriptor(const std::string& name,const std::string& type):
  m_name(name),m_type(type),m_classStores(){}

pool::ObjectRelationalContainerDescriptor::~ObjectRelationalContainerDescriptor(){
  for(std::vector<ObjectRelationalClassDescriptor*>::const_iterator iObj=m_classStores.begin();
      iObj!=m_classStores.end();iObj++){
    delete *iObj;
  }
}

const std::string& pool::ObjectRelationalContainerDescriptor::name() const {
  return m_name;
}

const std::string&  pool::ObjectRelationalContainerDescriptor::type() const {
  return m_type;
}


pool::ObjectRelationalClassDescriptor* pool::ObjectRelationalContainerDescriptor::addClassStore(const std::string& className, const std::string& mappingVersion){
  ObjectRelationalClassDescriptor* newStore = new ObjectRelationalClassDescriptor(className,mappingVersion);
  m_classStores.push_back(newStore);
  return newStore;
}

const std::vector<pool::ObjectRelationalClassDescriptor*>& pool::ObjectRelationalContainerDescriptor::classStores() const {
  return m_classStores;
}

void pool::ObjectRelationalContainerDescriptor::setRecreatePolicy(bool flag){
  m_recreate = flag;
}

bool pool::ObjectRelationalContainerDescriptor::recreatePolicy() const {
  return m_recreate;
}

std::string pool::ObjectRelationalContainerDescriptor::homogeneousContainerTypeName(){
  static std::string s_homogeneousContainerTypeName("homogeneous");
  return s_homogeneousContainerTypeName;
}

std::string pool::ObjectRelationalContainerDescriptor::polymorphicContainerTypeName(){
  static std::string s_polymorphicContainerTypeName("polymorphic");
  return s_polymorphicContainerTypeName;  
}

pool::ObjectRelationalDatabaseDescriptor::ObjectRelationalDatabaseDescriptor():m_containers(){}

pool::ObjectRelationalDatabaseDescriptor::~ObjectRelationalDatabaseDescriptor(){

}

pool::ObjectRelationalContainerDescriptor& pool::ObjectRelationalDatabaseDescriptor::addContainer(const std::string& containerName,const std::string& containerType){
  ObjectRelationalContainerDescriptor* cont = new ObjectRelationalContainerDescriptor(containerName,containerType);
  m_containers.push_back(cont);
  return *cont;
}

const std::vector<pool::ObjectRelationalContainerDescriptor*>& pool::ObjectRelationalDatabaseDescriptor::containers() const {
  return m_containers;
}

std::set<std::string> pool::ObjectRelationalDatabaseDescriptor::involvedClasses() const {
  std::set<std::string> allClasses;
  for(std::vector<pool::ObjectRelationalContainerDescriptor*>::const_iterator iCont=m_containers.begin();iCont!=m_containers.end();iCont++){
    for(std::vector<pool::ObjectRelationalClassDescriptor*>::const_iterator iObjStore=(*iCont)->classStores().begin();iObjStore!=(*iCont)->classStores().end();iObjStore++){
      allClasses.insert((*iObjStore)->className());
    }
  }
  return allClasses;
}


