#ifndef OBJECTRELATIONALACCESS_XMLDATABASEMANAGERDRIVER_H
#define OBJECTRELATIONALACCESS_XMLDATABASEMANAGERDRIVER_H

#include "ObjectRelationalAccess/ObjectRelationalDatabaseDescriptor.h"
#include "xercesc/sax2/SAX2XMLReader.hpp"
#include "xercesc/sax2/DefaultHandler.hpp"
#include "xercesc/sax/EntityResolver.hpp"
#include "xercesc/sax/InputSource.hpp"
#include <string>

#if defined( XERCES_HAS_CPP_NAMESPACE )
  using namespace xercesc;
#endif

XERCES_CPP_NAMESPACE_BEGIN
  class DTDGrammar;
XERCES_CPP_NAMESPACE_END

namespace pool {

  class ObjectRelationalMappingDescriptor;
  class ObjectRelationalMappingDescriptorElement;
  
  class XMLDatabaseManagerDriver {
 
  public:

    XMLDatabaseManagerDriver();

     virtual ~XMLDatabaseManagerDriver(){}

    ObjectRelationalDatabaseDescriptor* read(const std::string& fileName);

    ObjectRelationalDatabaseDescriptor* readFromBuffer(const char* buffer, const char* bufferId="InMemoryDocument");

    bool write(const ObjectRelationalDatabaseDescriptor& databaseStruct, std::ostream& os);
    
    //void printSetup(const std::string& fileName);

  private:

    void init();

    static std::string stringValue(const XMLCh* const xmlString);

    static void recursivePrint(const ObjectRelationalMappingDescriptorElement& mappingElement, DTDGrammar& dtdGrammar,
                               std::ostream& os, std::string margin="");

    static const std::string& databaseTag();

    static const std::string& containerTag();

    static const std::string& classTag();

    static const std::string& dependencyTag();

    static const std::string& nameAttribute();

    static const std::string& mappingVersionAttribute();

    static const std::string& containerTypeAttribute();

    static const std::string& homogeneousContainerTypeAttributeValue();

    static const std::string& policyAttribute();

    static const std::string& updatePolicyAttributeValue();

    class SAXHandler : public xercesc::DefaultHandler {
       public:

         SAXHandler();

         explicit SAXHandler(ObjectRelationalDatabaseDescriptor* dbSetup);

         virtual void startDocument();

         virtual void endDocument();

         virtual void startElement(const XMLCh* const uri,
                                   const XMLCh* const localname,
                                   const XMLCh* const qname,
                                   const Attributes&  attributes  );
    
         virtual void endElement( const XMLCh* const uri,
                                  const XMLCh* const localname,
                                  const XMLCh* const qname );

         virtual void error( const SAXParseException& exception );
 
         virtual void fatalError(const SAXParseException& exception);

     private:

         ObjectRelationalDatabaseDescriptor* m_dbSetup;
         ObjectRelationalContainerDescriptor* m_containerSetup;
         ObjectRelationalClassDescriptor* m_classStoreSetup;
         
         ObjectRelationalMappingDescriptor* m_mapping;
         ObjectRelationalMappingDescriptorElement* m_currentElement;

    };

    class SpecificEntityResolver : public xercesc::EntityResolver {
      public:
        SpecificEntityResolver() {}

        ~SpecificEntityResolver() {}

        InputSource* resolveEntity(const XMLCh* const publicId, const XMLCh* const systemId);
         
    };
  };

}

#endif // OBJECTRELATIONALACCESS_XMLDATABASEMANAGERDRIVER_H
