#include <iostream>
#include <stdexcept>
#include <memory>

#include "ObjectRelationalAccess/ObjectRelationalMapping.h"
#include "ObjectRelationalAccess/ObjectRelationalDefaultMappingBuilder.h"

#include "Reflex/SharedLibrary.h"
#include "Reflex/Type.h"
#include <string>

#include "RVersion.h"
#if ROOT_VERSION_CODE < ROOT_VERSION(5,19,0)
using namespace ROOT;
#endif


void
printMappingElement( const pool::ObjectRelationalMappingElement& element,
                     int level )
{
  std::cout << " ";
  for ( int l = 0; l < level; ++l ) std::cout << "  ";
  if ( ! element.scopeName().empty() ) std::cout << element.scopeName() << "::";
  std::cout << element.variableName()
            << " ( " << pool::ObjectRelationalMappingElement::elementTypeAsString( element.elementType() ) << " : " << element.variableType()
            << " ) -> " << element.tableName() << " ( ";
  for ( std::vector< std::string >::const_iterator c = element.columnNames().begin();
        c != element.columnNames().end(); ++c ) {
    if ( c != element.columnNames().begin() ) std::cout << ", ";
    std::cout << *c;
  }
  std::cout << " )" << std::endl;

  for ( pool::ObjectRelationalMappingElement::const_iterator i = element.begin();
        i != element.end(); ++i ) {
    printMappingElement( i->second, level + 1 );
  }
}


int main( int, char** )
{
  try {

    Reflex::SharedLibrary libloader( "SealSTLRflx" );

    if(!libloader.Load()){
      std::cout << libloader.Error() << std::endl;;
    }


    pool::ObjectRelationalDefaultMappingBuilder builder;
    pool::ObjectRelationalMapping mapping( "version_x" );

    ROOT::Reflex::Type dictionary = ROOT::Reflex::Type::ByTypeInfo( typeid(std::string) );
    if(!dictionary) std::cout << "Lookup by typeid for string  did not work..." <<std::endl; 
    dictionary = ROOT::Reflex::Type::ByName("std::string");
    if(!dictionary) std::cout << "Lookup by name for string  did not work..." <<std::endl; 

    if ( ! dictionary ) {
      throw std::runtime_error( "Dictionary not found" );
    }
    if ( ! builder.processClass( mapping, dictionary ) ) {
      throw std::runtime_error( "Could not build the default mapping" );
    }

    printMappingElement( mapping.element(), 0 );

    std::cout << "Exiting..." << std::endl;
  }
  catch ( std::exception& e ) {
    std::cerr << "Standard C++ exception : " << e.what() << std::endl;
    return 1;
  }
  catch ( ... ) {
    std::cerr << "Unknown exception ... " << std::endl;
    return 1;
  }
  return 0;
}
