#include <iostream>
#include <stdexcept>
#include <memory>
#include <cstdlib>

#include "RelationalAccess/ConnectionService.h"
#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/ITransaction.h"
#include "ObjectRelationalAccess/ObjectRelationalMappingPersistency.h"
#include "TestDriver.h"

int main( int, char** )
{
  try {
    // Set the authentication and lookup paths
    if ( ! ::getenv( "CORAL_AUTH_PATH" ) ) {
      char my_arg[] = "CORAL_AUTH_PATH=/afs/cern.ch/sw/lcg/app/pool/db";       
      ::putenv( my_arg );
    }
    if ( ! ::getenv( "CORAL_DBLOOKUP_PATH" ) ) {
      char my_arg[] = "CORAL_DBLOOKUP_PATH=/afs/cern.ch/sw/lcg/app/pool/db";       
      ::putenv( my_arg );
    }
    const std::string connectionString = "/pool/db/service_1";

    coral::ConnectionService connService;

    std::cout << "Connecting..." << std::endl;
    coral::ISessionProxy* session = connService.connect( connectionString );

    TestDriver driver;
    
    // removing old mappings
    session->transaction().start(); 
    driver.removeOldMappings( session->nominalSchema() );
    session->transaction().commit();

    // writing the mappings
    session->transaction().start();
    driver.writeMapping( session->nominalSchema() );
    session->transaction().commit();

    // reading back a mapping
    session->transaction().start();
    driver.readMapping( session->nominalSchema() );
    session->transaction().commit();

    std::cout << "Disconnecting..." << std::endl;
    delete session;
    std::cout << "Exiting..." << std::endl;
  }
  catch ( std::exception& e ) {
    std::cerr << "Standard C++ exception : " << e.what() << std::endl;
    return 1;
  }
  catch ( ... ) {
    std::cerr << "Unknown exception ... " << std::endl;
    return 1;
  }
  return 0;
}
