#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/ITransaction.h"
#include "POOLCore/Exception.h"
#include "POOLCore/CommandOptions.h"
#include "ObjectRelationalAccess/ObjectRelationalMappingUtilities.h"
#include <iostream>

int main (int argc, char** argv)
{
  std::vector<pool::CommandOptions::Option> ops;
  pool::CommandOptions::Option opt;
  opt.flag = "-c";
  opt.property = "connectionString";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the destination connection string";
  opt.required = true;
  ops.push_back(opt);
  opt.flag = "-u";
  opt.property = "user";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the user name of the destination account";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-p";
  opt.property = "password";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the user password of the destination account";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-s";
  opt.property = "sourceConnectionString";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the source connection string";
  opt.required = true;
  ops.push_back(opt);
  opt.flag = "-su";
  opt.property = "sourceUser";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the user name of the source account";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-sp";
  opt.property = "sourcePassword";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the user password of the source account";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-n";
  opt.property = "containerName";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the selected container name";
  opt.required = true;
  ops.push_back(opt);  
  opt.flag = "-cv";
  opt.property = "classVersion";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the class version";
  opt.required = false;
  ops.push_back(opt);  
  opt.flag = "-d";
  opt.property = "dictionary";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the dictionary libraries";
  opt.required = false;
  ops.push_back(opt);  
  opt.flag = "-all";
  opt.property = "allVersions";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.required = false;
  opt.helpEntry = "retrieve all available versions";
  ops.push_back(opt);   
  opt.flag = "-m";
  opt.property = "materialize";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.required = false;
  opt.helpEntry = "materialize the exported tables";
  ops.push_back(opt);   
  opt.flag = "-debug";
  opt.property = "debug";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.required = false;
  opt.helpEntry = "enable the debug verbosity mode";
  ops.push_back(opt);   
  opt.flag = "-info";
  opt.property = "info";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.required = false;
  opt.helpEntry = "enable the info verbosity mode";
  ops.push_back(opt);   
  opt.flag = "-h";
  opt.property = "help";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.helpEntry = "display the help";
  opt.required = false;
  ops.push_back(opt);
  pool::CommandOptions cmd(ops);
  try{
    cmd.parse(argc,argv);
    const std::map<std::string,std::string>& ops = cmd.userOptions();
    std::map<std::string,std::string>::const_iterator iter = ops.find("help");
    if(iter!=ops.end() || ops.size()==0){
      cmd.help();
    } else {
      pool::ObjectRelationalMappingUtilities utility;
      iter = ops.find("debug");
      if(iter!=ops.end()){
        utility.setProperty(pool::ObjectRelationalMappingUtilities::verbosityProperty(),
                            pool::ObjectRelationalMappingUtilities::debugVerbosityPropertyValue());
      } else {
        iter = ops.find("info");
        if(iter!=ops.end()){
          utility.setProperty(pool::ObjectRelationalMappingUtilities::verbosityProperty(),
                              pool::ObjectRelationalMappingUtilities::infoVerbosityPropertyValue());
        }
      }
      std::string theMappingOutFile = "mapping.xml";
      iter = ops.find("outputfile");
      if(iter!=ops.end()){
        theMappingOutFile = iter->second;
      }

      std::string connectionString("");
      iter = ops.find("sourceConnectionString");
      if(iter!=ops.end()){
        connectionString = iter->second;
      }
      std::string userName("");
      iter = ops.find("sourceUser");
      if(iter!=ops.end()){
        userName = iter->second;
        utility.setProperty(pool::ObjectRelationalMappingUtilities::userNameProperty(),userName);
      }
      std::string password = (*iter).second;
      iter = ops.find("sourcePassword");
      if(iter!=ops.end()){
        password = (*iter).second;
        utility.setProperty(pool::ObjectRelationalMappingUtilities::passwordProperty(),password);
      }
      std::string destConnectionString("");
      iter = ops.find("connectionString");
      if(iter!=ops.end()){
        destConnectionString = iter->second;
      }
      std::string destUserName("");
      iter = ops.find("user");
      if(iter!=ops.end()){
        destUserName = iter->second;
      }
      std::string destPassword = (*iter).second;
      iter = ops.find("password");
      if(iter!=ops.end()){
        destPassword = (*iter).second;
      }
      std::string contName("");
      iter = ops.find("containerName");
      if(iter!=ops.end()){
        contName = (*iter).second;
      }
      if( contName.empty()){
        std::cout << "ERROR: container name not provided." << std::endl;
        return 1;        
      }
      
      std::string classVersion("");
      iter = ops.find("classVersion");
      if(iter!=ops.end()){
        classVersion = (*iter).second;
      }
      iter = ops.find("dictionary");
      if(iter!=ops.end()){
        std::string theDictionaryLibraries = (*iter).second;
        utility.loadDictionaries(theDictionaryLibraries);
      }
      bool allVersions = false;
      iter = ops.find("allVersions");
      if(iter!=ops.end()){
        allVersions = true;
      }
      bool materialize = false;
      iter = ops.find("materialize");
      if(iter!=ops.end()){
        materialize = true;
      }

      if( !allVersions && contName.empty() ){
        throw pool::Exception("Container name not provided", "", "pool_export_object_relational_mapping" );        
      }
      
      utility.initialize();
      std::cout << "Connecting to source database \""<<connectionString<<"\"."<<std::endl;
      utility.connect(connectionString,coral::ReadOnly);
      utility.session()->transaction().start(true);
      try{
        if(allVersions && !classVersion.empty())std::cout << "Warning: Class version selection will be ignored." << std::endl;
        utility.loadMappingInformation( contName, classVersion, allVersions);
        utility.session()->transaction().commit();
        std::cout << "Building mapping into destination database..." << theMappingOutFile << std::endl;
        utility.setProperty(pool::ObjectRelationalMappingUtilities::userNameProperty(),destUserName);
        utility.setProperty(pool::ObjectRelationalMappingUtilities::passwordProperty(),destPassword);
        std::cout << "Connecting to destination database \""<<destConnectionString<<"\"."<<std::endl;
        utility.connect(destConnectionString,coral::Update);
        utility.session()->transaction().start();
        bool okStore = utility.storeMappingInformation( materialize );
        utility.session()->transaction().commit();
        if(okStore) {
          std::cout << "Mapping exported." << std::endl;
        } else {
          std::cout << "Could not export mapping." << std::endl;
        }
      } catch (const std::exception& e){
        utility.session()->transaction().rollback();
        std::cout << "ERROR: " << e.what() << std::endl;
        return 1;
      }
    }
    return 0;
  } catch (const std::exception& e){
    std::cout << "ERROR: " << e.what() << std::endl;
    return 1;
  }
}

