#ifndef PERSISTENCYSVC_ICONFIGURATION_H
#define PERSISTENCYSVC_ICONFIGURATION_H

#include "POOLCore/Context.h"
#include "PersistencySvc/DatabaseConnectionPolicy.h"

namespace pool {

  // forward declarations
  class IFileCatalog;
  class ITokenValidator;
  class IDataTransform;

    /** @class IConfiguration
     *
     *  IConfiguration holds the file catalog reference,
     *  the token validator, the data transform and database connection policy objects,
     *  that are used in the various operations of the PersistencySvc classes.
     *
     */
  class IConfiguration : virtual public Context {
    public:
      /// Constructor. 
      IConfiguration();
      
      /// Destructor
      virtual ~IConfiguration(){}
      
      /** Sets the attached file catalog
       *
       *  @param catalog   [IN] Reference to the file catalog
       */
      virtual void setFileCatalog( IFileCatalog& catalog ) = 0;
      
      /// Retrieves the file catalog in use.
      virtual IFileCatalog& fileCatalog() = 0;
      virtual const IFileCatalog& fileCatalog() const = 0;
      
      /** Sets the attached object token validator
       *
       *  @param validator [IN] Reference to the external validator
       */
      virtual void setTokenValidator( ITokenValidator& validator ) = 0;
      
      /// Retrieves the token valitator in use
      virtual ITokenValidator& tokenValidator() = 0;
      
      /** Sets the attached object data transform
       *
       *  @param transform [IN] Reference to the external data transform
       */
      virtual void setDataTransform( IDataTransform& transform ) = 0;
      
      /// Returns a reference to the IDataTransform object.
      virtual IDataTransform& dataTransform() = 0;

      /// Sets the database connection policy
      virtual void setDatabaseConnectionPolicy( const DatabaseConnectionPolicy& policy ) = 0;
      
      /// Returns the database connection policy.
      virtual const DatabaseConnectionPolicy& databaseConnectionPolicy() const = 0;
  };
}

inline
pool::IConfiguration::IConfiguration():Context(){
}

#endif
