#ifndef POOL_IDATATRANSFORM_H
#define POOL_IDATATRANSFORM_H

#include "PersistencySvc/IShapeTransformationRegistry.h"

#include "RVersion.h"
#if ROOT_VERSION_CODE < ROOT_VERSION(5,19,0)
namespace ROOT { namespace Reflex   {  class Type;  } }
#else
namespace Reflex   {  class Type;  } 
#endif

/** 
    pool namespace.
*/
#if ROOT_VERSION_CODE < ROOT_VERSION(5,19,0)
using namespace ROOT;
#endif

namespace pool  {

  class Guid;


  /** class IDataTransform.
   *  Abstract class for the management of the callback objects
   *  for the transient and persistent shapes.
   */
  class IDataTransform : public IShapeTransformationRegistry {
  public:
    /// Constructor
    IDataTransform() {}

    /// Destructor
    virtual ~IDataTransform() {};

    /**
      Sets the behaviour for acting when a callback object is not found.
      If set, a missing callback object is created automatically (default behaviour),
      else an exception is thrown.
    */
    virtual void setAutomaticCallBackCreation( bool flag = true ) = 0;


    /// Access Transformation object to convert an object.
    /**
        @param   object_type    [IN] Type of the transient object
        @param   shape_type  [IN] Type of the persistent shape.
        @return DataCallBack&   Reference to the callback object.
    */
    virtual DataCallBack& transformation( const pool::Guid& object_type,
                                          const Reflex::Type& shape_type ) = 0;
    virtual DataCallBack& transformation( const Reflex::Type& object_type,
                                          const Reflex::Type& shape_type ) = 0;
  };
}
#endif // POOL_IDATATRANSFORM_H
