#ifndef POOL_ISHAPETRANSFORMATIONREGISTRY_H
#define POOL_ISHAPETRANSFORMATIONREGISTRY_H

#include "RVersion.h"

#if ROOT_VERSION_CODE < ROOT_VERSION(5,19,0)
namespace ROOT { namespace Reflex { class Type; } }
#else
namespace Reflex { class Type; } 
#endif

/** 
    pool namespace.
*/
#if ROOT_VERSION_CODE < ROOT_VERSION(5,19,0)
using namespace ROOT;
#endif

namespace pool  {
  class Guid;
  class DataCallBack;

  /** class IShapeTransformationRegistry
   *  Abstract class for the registring of callback objects
   *  for the transient and persistent shapes.
   */
  class IShapeTransformationRegistry {
  public:
    virtual ~IShapeTransformationRegistry() {}

    /// Register a new transformation object. Returns false if a transformation already exists, true otherwise.
    virtual bool registerTransformation( const pool::Guid& object_type, 
                                         const Reflex::Type& shape_type,
                                         const DataCallBack& callback ) = 0;

    /// Register a new transformation object. Returns false if a transformation already exists, true otherwise.
    virtual bool registerTransformation( const Reflex::Type& object_type, 
                                         const Reflex::Type& shape_type,
                                         const DataCallBack& callback ) = 0;

    /// Deregisters an existing transformation object. Returns false if a transformation does not exist, true otherwise.
    virtual bool deregisterTransformation( const pool::Guid& object_type,
                                           const Reflex::Type& shape_type ) = 0;

    /// Deregisters an existing transformation object. Returns false if a transformation does not exist, true otherwise.
    virtual bool deregisterTransformation( const Reflex::Type& object_type,
                                           const Reflex::Type& shape_type ) = 0;
  };
}
#endif // POOL_ISHAPETRANSFORMATIONREGISTRY_H
