#ifndef PERSISTENCYSVC_DATATRANSFORM_H
#define PERSISTENCYSVC_DATATRANSFORM_H

#include "PersistencySvc/IDataTransform.h"
#include "Reflex/Type.h"
#include "POOLCore/Guid.h"
#include <map>

#include "RVersion.h"

namespace pool {
  namespace PersistencySvc {

#if ROOT_VERSION_CODE < ROOT_VERSION(5,19,0)
    typedef ROOT::Reflex::Type TypeH;
#else
    typedef::Reflex::Type TypeH;
#endif

    class DataTransform : public virtual IDataTransform {
    public:
      DataTransform();
      ~DataTransform();
      
      // Methods from IDataTransform that have to be implemented
      void setAutomaticCallBackCreation( bool flag = true );

      DataCallBack& transformation( const pool::Guid& object_type,
                                    const TypeH& shape_type );

      DataCallBack& transformation( const TypeH& object_type,
                                    const TypeH& shape_type );

      bool registerTransformation( const pool::Guid& object_type,
                                   const TypeH& shape_type,
                                   const DataCallBack& callback );

      bool registerTransformation( const TypeH& object_type,
                                   const TypeH& shape_type,
                                   const DataCallBack& callback );

      bool deregisterTransformation( const pool::Guid& object_type, 
                                     const TypeH& shape_type );

      bool deregisterTransformation( const TypeH& object_type, 
                                     const TypeH& shape_type );
    private:
      bool m_automaticCreationFlag;
      std::map< pool::Guid, std::map< TypeH, DataCallBack* > > m_callBackObjects;
      std::map< TypeH, std::map< TypeH, DataCallBack* > > m_callBackObjectsTransientType;
    };
  }
}

#endif
