#include "PersistencySvcConfiguration.h"
#include "FileCatalog/IFileCatalog.h"
#include "DummyTokenValidator.h"
#include "DataTransform.h"

pool::PersistencySvc::PersistencySvcConfiguration::PersistencySvcConfiguration():
  m_fileCatalog( 0 ),
  m_tokenValidator( new pool::PersistencySvc::DummyTokenValidator ),
  m_ownTokenValidator( true ),
  m_dataTransform( new pool::PersistencySvc::DataTransform ),
  m_ownDataTransform( true )
{}
    
pool::PersistencySvc::PersistencySvcConfiguration::~PersistencySvcConfiguration()
{
  if ( m_ownTokenValidator ) delete m_tokenValidator;
  if ( m_ownDataTransform ) delete m_dataTransform;
}
      
void
pool::PersistencySvc::PersistencySvcConfiguration::setFileCatalog( pool::IFileCatalog& catalog )
{
  m_fileCatalog = &catalog;
}

pool::IFileCatalog&
pool::PersistencySvc::PersistencySvcConfiguration::fileCatalog()
{
  return *m_fileCatalog;
}

const pool::IFileCatalog&
pool::PersistencySvc::PersistencySvcConfiguration::fileCatalog() const
{
  return *m_fileCatalog;
}

void
pool::PersistencySvc::PersistencySvcConfiguration::setTokenValidator( pool::ITokenValidator& validator )
{
  if ( m_ownTokenValidator ) {
    delete m_tokenValidator;
    m_ownTokenValidator = false;
  }
  m_tokenValidator = &validator;
}

pool::ITokenValidator&
pool::PersistencySvc::PersistencySvcConfiguration::tokenValidator()
{
  return *m_tokenValidator;
}

void
pool::PersistencySvc::PersistencySvcConfiguration::setDataTransform( pool::IDataTransform& transform )
{
  if ( m_ownDataTransform ) {
    delete m_dataTransform;
    m_ownDataTransform = false;
  }
  m_dataTransform = &transform;
}

pool::IDataTransform&
pool::PersistencySvc::PersistencySvcConfiguration::dataTransform()
{
  return *m_dataTransform;
}

void
pool::PersistencySvc::PersistencySvcConfiguration::setDatabaseConnectionPolicy( const pool::DatabaseConnectionPolicy& policy )
{
  m_policy = policy;
}

const pool::DatabaseConnectionPolicy&
pool::PersistencySvc::PersistencySvcConfiguration::databaseConnectionPolicy() const
{
  return m_policy;
}
