#ifndef BASECONTAINER_HANDLER_H
#define BASECONTAINER_HANDLER_H

#include "IContainerHandler.h"
#include "Reflex/Reflex.h"
#include "Reflex/Builder/CollectionProxy.h"
#include "RVersion.h"

#if ROOT_VERSION_CODE < ROOT_VERSION(5,19,0)
 typedef ROOT::Reflex::Type TypeH;
 using namespace ROOT;
#else
 typedef Reflex::Type TypeH;
#endif

namespace pool {
 
  namespace RelationalStorageService {

    class BaseContainerHandler : virtual public IContainerHandler {

    public:
      /// Constructor
      explicit BaseContainerHandler( const TypeH& dictionary );

      /// Destructor
      ~BaseContainerHandler();

      /// Returns the size of the container
      size_t size( const void* address );

      /// Returns an initialized iterator
      IContainerIteratorHandler* iterate( const void* address );

      /// Appends a new element and returns its address of the object reference
      void appendNewElement( void* address, void* data );

      /// Clear the content of the container
      void clear( const void* address );
      
      /// Returns the iterator return type
      TypeH iteratorReturnType() const;

      /// Returns the associativeness of the container
      bool isAssociative() const { return m_isAssociative; }
      
    private:
      /// The dictionary information
      TypeH m_type;

      /// The iterator return type
      TypeH m_iteratorReturnType;

      /// Flag indicating whether the container is associative
      bool m_isAssociative;

      /// Structure containing parameters of the collection instance  
      Reflex::Environ<long> m_collEnv;

      /// Proxy of the generic collection
      Reflex::CollFuncTable* m_collProxy;

    };

  }

}

#endif
