#ifndef CARRAY_HANDLER_H
#define CARRAY_HANDLER_H

#include "IContainerHandler.h"
#include "Reflex/Reflex.h"
#include "RVersion.h"

namespace pool {

#if ROOT_VERSION_CODE < ROOT_VERSION(5,19,0)
 typedef ROOT::Reflex::Type TypeH;
#else
 typedef Reflex::Type TypeH;
#endif

  namespace RelationalStorageService {

    class CArrayHandler : virtual public IContainerHandler {

    public:
      /// Constructor
      explicit CArrayHandler( const TypeH& dictionary );

      /// Destructor
      ~CArrayHandler();

      /// Returns the size of the container
      size_t size( const void* address );

      /// Returns an initialized iterator
      IContainerIteratorHandler* iterate( const void* address );

      /// Appends a new element and returns its address of the object reference
      void appendNewElement( void* address, void* data );

      /// Clear the content of the container
      void clear( const void* address );
      
      /// Returns the iterator return type
      TypeH iteratorReturnType() const;

      /// Returns the associativeness of the container
      bool isAssociative() const { return false; }
      
    private:
      /// The dictionary information
      TypeH m_type;

      /// The iterator return type
      TypeH m_elementType;
    };

  }

}

#endif
