#include "ContainerHandlerFactory.h"
#include "BaseContainerHandler.h"
#include "SpecialContainerHandler.h"
#include "CArrayHandler.h"
#include "PVectorContainerHandler.h"
#include "Reflex/Type.h"


pool::RelationalStorageService::IContainerHandler*
pool::RelationalStorageService::ContainerHandlerFactory::newContainerHandler( const Reflex::Type& containerType )
{
  if(containerType.IsArray()){
    return new pool::RelationalStorageService::CArrayHandler( containerType );
  } else {  
    Reflex::TypeTemplate templ = containerType.TemplateFamily();
    if ( templ ) {
      std::string contName = templ.Name(Reflex::SCOPED|Reflex::FINAL); 
      if(  contName == "std::vector"              ||
           contName == "std::list"                ||
           contName == "std::set"                 ||
           contName == "std::multiset"            ||
           contName == "std::deque"               ||
           contName == "__gnu_cxx::hash_set"      ||
           contName == "__gnu_cxx::hash_multiset" ||
           contName == "std::map"                 ||
           contName == "std::multimap"            ||
           contName == "__gnu_cxx::hash_map"      ||
           contName == "__gnu_cxx::hash_multimap" ){
        return new pool::RelationalStorageService::BaseContainerHandler( containerType );
      } else if (  contName == "std::stack"      ||
                   contName == "std::queue"      ){
        return new pool::RelationalStorageService::SpecialContainerHandler( containerType );
      } else if (  contName == "pool::PVector" ) {
        return new pool::RelationalStorageService::PVectorContainerHandler( containerType );
      }
      
    }
  }
  return 0;
}

