#ifndef RELATIONALSTORAGESERVICE_MAPPING_MANAGER_H
#define RELATIONALSTORAGESERVICE_MAPPING_MANAGER_H

#include <string>
#include <map>
#include "RVersion.h"

namespace coral {

  class ISchema;
  
}

#if ROOT_VERSION_CODE < ROOT_VERSION(5,19,0)
namespace ROOT{
  namespace Reflex{
    class Type;
  }
}
using namespace ROOT;
#else
namespace Reflex{
  class Type;
}
#endif 

namespace pool {
  
  class ObjectRelationalMappingPersistency;
  class ObjectRelationalMapping;
  class ObjectRelationalMappingElement;

  namespace RelationalStorageService {

    /**
       @class MappingManager MappingManager.h
       Utility class to manage the object-relational mappings for the C++ classes.
    */

    class MappingManager {

      public:
      /// Constructor
      MappingManager( const std::string& containerName, coral::ISchema& schema );

      /// Destructor
      ~MappingManager();

      /// Retrieves a valid mapping for a given class.
      const ObjectRelationalMapping& mappingForClass( const Reflex::Type& dictionaryEntry, bool writingEnabled=false );

      /// Retrieves a valid mapping for a given depending class.
      const ObjectRelationalMappingElement& mappingForDependingClass( const Reflex::Type& classDictionaryEntry,
                                                                      const Reflex::Type& parentDictionaryEntry,
                                                                      bool writingEnabled=false  );
      
      /// Clears the internal state
      void clear();
      
      public:

      static std::string versionOfClass( const Reflex::Type& dictionary );

      private:
      
      /// The schema in use
      coral::ISchema& m_schema;
      /// The concerned container
      std::string m_containerName;
      /// The mapping persistency service
      pool::ObjectRelationalMappingPersistency* m_persistency;
      mutable std::map< std::string, std::string > m_mappingVersionsForClass;
      /// The map of mapping information objects
      mutable std::map< std::string, std::map< std::string, ObjectRelationalMapping* > > m_mappings;
    };

  }

}

#endif
