#include "POOLCore/CommandOptions.h"
#include "RelationalStorageService/RelationalPoolDatabaseUtilities.h"

#include <exception>
#include <iostream>
#include <iomanip>
#include <sstream>

int main (int argc, char** argv)
{
  std::vector<pool::CommandOptions::Option> ops;
  pool::CommandOptions::Option opt;
  opt.flag = "-c";
  opt.property = "connectionString";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the connection string";
  opt.required = true;
  ops.push_back(opt);
  opt.flag = "-u";
  opt.property = "user";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the user name";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-p";
  opt.property = "password";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the user password";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-d";
  opt.property = "dictionary";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the dictionary libraries";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-debug";
  opt.property = "debug";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.required = false;
  opt.helpEntry = "enable the debug verbosity mode";
  ops.push_back(opt);   
  opt.flag = "-info";
  opt.property = "info";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.required = false;
  opt.helpEntry = "enable the info verbosity mode";
  ops.push_back(opt);   
  opt.flag = "-h";
  opt.property = "help";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.helpEntry = "display the help";
  opt.required = false;  
  ops.push_back(opt);
  pool::CommandOptions cmd(ops);
  try{
    cmd.parse(argc,argv);
    const std::map<std::string,std::string>& ops = cmd.userOptions();
    std::map<std::string,std::string>::const_iterator iter = ops.find("help");
    if(iter!=ops.end() || ops.size()==0){
      cmd.help();
    } else {
      pool::RelationalPoolDatabaseUtilities utility;
    
      iter = ops.find("debug");
      if(iter!=ops.end()){
        utility.setProperty(pool::ObjectRelationalUtilities::verbosityProperty(),
                            pool::ObjectRelationalUtilities::debugVerbosityPropertyValue());
      } else {
        iter = ops.find("info");
        if(iter!=ops.end()){
          utility.setProperty(pool::ObjectRelationalUtilities::verbosityProperty(),
                              pool::ObjectRelationalUtilities::infoVerbosityPropertyValue());
        }
      }
      iter = ops.find("dictionary");
      if(iter!=ops.end()){
        std::string theDictionaryLibraries = (*iter).second;
        utility.loadDictionaries(theDictionaryLibraries);
      } 

      iter = ops.find("connectionString");
      std::string theConnectionString("");
      if(iter!=ops.end()){
        theConnectionString = (*iter).second;
        utility.setProperty(pool::ObjectRelationalUtilities::connectionStringProperty(),theConnectionString);
      } 
      iter = ops.find("user");
      if(iter!=ops.end()){
        std::string theUserName = (*iter).second;
        utility.setProperty(pool::ObjectRelationalUtilities::userNameProperty(),theUserName);
      } 
      iter = ops.find("password");
      if(iter!=ops.end()){
        std::string thePassword = (*iter).second;
        utility.setProperty(pool::ObjectRelationalUtilities::passwordProperty(),thePassword);
      } 
      pool::RelationalPoolDatabaseUtilities::DatabaseInfo dbInfo = utility.queryDatabase();
      if(dbInfo.connectionString!=""){
        unsigned int sz = dbInfo.containers.size();
        if(sz){
          std::cout << "Database " <<dbInfo.connectionString << ": "<< sz << " container(s) found." << std::endl;
          std::string containerName("Cont.Name");
          std::string tech("Technology");
          std::string clName("Class");
          std::string mapVersion("Map.Version");
          std::string entries("Entries");
          size_t contMax= containerName.length();
          size_t techMax = tech.length();
          size_t clMax = clName.length();
          size_t mapVersMax = mapVersion.length();
          size_t entriesMax = entries.length();
          
          // first find the max lenghts
          std::map<std::string,pool::RelationalPoolDatabaseUtilities::ContainerInfo>::const_iterator iCont;
          for(iCont=dbInfo.containers.begin();iCont!=dbInfo.containers.end();iCont++){
            if(iCont->second.name.length()>contMax) contMax = iCont->second.name.length();
            if(iCont->second.technology.length()>techMax) techMax = iCont->second.technology.length();
            if(iCont->second.mappingVersion.length()>mapVersMax) mapVersMax = iCont->second.mappingVersion.length();
            if(iCont->second.className.length()>clMax) clMax = iCont->second.className.length();
            std::ostringstream entrs;
            entrs << iCont->second.size;
            if(entrs.str().length()>entriesMax) entriesMax = entrs.str().length();
          }
          std::string space("  ");
          std::cout << std::setiosflags(std::ios_base::left);
          std::cout << space << std::setw(contMax)<<containerName;
          std::cout << space << std::setw(techMax)<<tech;
          std::cout << space << std::setw(clMax)<<clName;
          std::cout << space << std::setw(mapVersMax)<<mapVersion;
          std::cout << space << std::setw(entriesMax)<<entries;
          std::cout << std::endl;
          std::cout << space << std::setfill('-')<<std::setw(contMax)<<"";
          std::cout << space << std::setw(techMax)<<"";
          std::cout << space << std::setw(clMax)<<"";
          std::cout << space << std::setw(mapVersMax)<<"";
          std::cout << space << std::setw(entriesMax)<<"";          
          std::cout << std::endl;
          std::cout << std::setfill(' '); 
          for(iCont=dbInfo.containers.begin();iCont!=dbInfo.containers.end();iCont++){
            std::cout << space << std::setw(contMax)<<iCont->second.name;
            std::cout << space << std::setw(techMax)<<iCont->second.technology;
            std::cout << space << std::setw(clMax)<<iCont->second.className;
            std::cout << space << std::setw(mapVersMax)<<iCont->second.mappingVersion;
            std::cout << space << std::setw(entriesMax)<<iCont->second.size;
            std::cout << std::endl;
            std::map<std::string, unsigned int>::const_iterator iClass;
            for(iClass=iCont->second.classes.begin();iClass!=iCont->second.classes.end();iClass++){
              std::cout << space << std::setw(contMax)<<"";
              std::cout << space << std::setw(techMax)<<"";
              std::cout << space << std::setw(clMax)<<iClass->first;
              std::cout << space << std::setw(mapVersMax)<<"";
              std::cout << space << std::setw(entriesMax)<<iClass->second;
              std::cout << std::endl;
            }
          }
        } else {
          std::cout << "Database " <<theConnectionString << ": no containers found." << std::endl;
        }
      } else {
        std::cout << "POOL Database not found on "<<theConnectionString<<std::endl;
      }
    }
    return 0;
  } catch (const std::exception& e){
    std::cout << "ERROR: " << e.what() << std::endl;
    return 1;
  }
}

