// $Id: DbDefaultDataHandler.cpp,v 1.18 2007/11/20 16:43:09 frankb Exp $
//====================================================================
//  DbCallBack object implementation
//--------------------------------------------------------------------
//
//  Package    : System (The POOL project)
//
//  Description: Generic data persistency
//
//  @author      M.Frank
//====================================================================

#include "StorageSvc/DataCallBack.h"
#include "POOLCore/Reference.h"
#include "StorageSvc/DbToken.h"
#include "StorageSvc/DbDatabase.h"
#include "StorageSvc/DbDefaultDataHandler.h"
using namespace pool;

/// Standard constructor
DbDefaultDataHandler::DbDefaultDataHandler() : m_ioContext(0) {
}

/// Standard destructor: release used resources
DbDefaultDataHandler::~DbDefaultDataHandler()   {
}

/// Callback to handle on the fly object manipulation during save operations.
DbStatus DbDefaultDataHandler::onSave(int       data_type,
                                      TypeH  /* reflection */,
                                      void*     object_ptr,
                                      void*     data)
{
  switch( data_type )   {
  case DataCallBack::PRIMITIVE:
    return Success;
  case DataCallBack::OBJECT:
    return saveObject(object_ptr, data);
  case DataCallBack::TOKEN:
    return saveToken((Token*)object_ptr);
  case DataCallBack::REFERENCE:
    return saveReference((Reference*)object_ptr, (Token::OID_t*)data);
  default:
    return Success;
  }
}

/// Callback to handle on the fly object manipulation during load operations.
DbStatus DbDefaultDataHandler::onLoad( int       data_type,
                                       TypeH  /* reflection */,
                                       void*     object_ptr,
                                       void*     data)
{
  switch( data_type )   {
  case DataCallBack::PRIMITIVE:
    return Success;
  case DataCallBack::OBJECT:
    return loadObject(object_ptr, data);
  case DataCallBack::REFERENCE:
    return loadReference((Reference*)object_ptr, (Token::OID_t*)data);
  case DataCallBack::TOKEN:
    return loadToken((Token*)object_ptr);
  default:
    return Success;
  }
}

/// Internal call on saving the token of a given reference
DbStatus DbDefaultDataHandler::saveToken(const Token* pToken, Token::OID_t* oid) {
  if ( m_ioContext && pToken && oid )  {
    DbDatabase* pdb = (DbDatabase*)m_ioContext;
    if ( pdb->isValid() )  {
      return pdb->makeLink(pToken, *oid);
    }
  }
  oid->first = oid->second = ~0x0;
  return Error;
}

/// Internal call to create the token of a given reference
DbStatus DbDefaultDataHandler::createToken(Token::OID_t* oid, Token*& pToken) {
  if ( m_ioContext && oid )  {
    DbDatabase* pdb = (DbDatabase*)m_ioContext;
    if ( pdb->isValid() )  {
      Token* tok = new Token();
      DbStatus sc = pdb->getLink(*oid, tok);
      if ( sc.isSuccess() )  {
        pToken = tok;
        return Success;
      }
      tok->release();
    }
  }
  pToken = 0;
  return Error;
}

/// Internal call on saving references
DbStatus DbDefaultDataHandler::saveReference(Reference* /* pRef */, Token::OID_t* /* oid */) {
  return Success;
}

/// Internal call on loading references
DbStatus DbDefaultDataHandler::loadReference(Reference* /* pRef */, Token::OID_t* /* oid  */) {
  return Success;
}

/// Internal call on saving tokens
DbStatus DbDefaultDataHandler::saveToken(Token* pToken)  {
  return pToken ? saveToken(pToken, &pToken->oid()) : Error;
}

/// Internal call on loading tokens
DbStatus DbDefaultDataHandler::loadToken(Token* pToken) {
  if ( m_ioContext && pToken )  {
    DbDatabase* pdb = (DbDatabase*)m_ioContext;
    if ( pdb->isValid() )  {
      DbStatus sc = pdb->getLink(pToken->oid(), pToken);
      if ( sc.isSuccess() )  {
        return Success;
      }
    }
  }
  return Error;
}

/// Internal call on saving objects
DbStatus DbDefaultDataHandler::saveObject(void* /* pObj */, void* /* data */ )  {
  return Success;
}

/// Internal call on loading objects
DbStatus DbDefaultDataHandler::loadObject(void* /* pObj */, void* /* data */ )  {
  return Success;
}
