// $Id: DbReflex.cpp,v 1.25 2008/10/21 08:39:59 frankb Exp $
//====================================================================
//  Package    : StorageSvc (The POOL project)
//
//  @author      M.Frank
//====================================================================
#include "StorageSvc/DbReflex.h"
#include "StorageSvc/DbTypeInfo.h"
#include "StorageSvc/DbTransform.h"
#include "StorageSvc/DbPrint.h"
#include <cstring>

#if ROOT_VERSION_CODE < ROOT_VERSION(5,19,0)
using namespace ROOT;
#endif
using namespace Reflex;
using namespace pool;
using namespace std;

namespace pool {  void genMD5(const string& s, void* code);  }

namespace {
  void normalizeGuidString(string& s)  {
    for(char* c = (char*)s.c_str(); *c; ++c)  {
      switch(*c) {
        case 'a':  *c = 'A'; break;
        case 'b':  *c = 'B'; break;
        case 'c':  *c = 'C'; break;
        case 'd':  *c = 'D'; break;
        case 'e':  *c = 'E'; break;
        case 'f':  *c = 'F'; break;
      }
    }
  }
}

typedef map<Guid,Type> GuidMap;
static GuidMap& guid_mapping()  {
  static GuidMap s_map;
  return s_map;
}

typedef map<Type,Guid> TypeMap;
static TypeMap& type_mapping()  {
  static TypeMap s_map;
  return s_map;
}

/// Dump all known dictionaries
void DbReflex::dumpDictionaries()  {
  string n;
  for(size_t i=0; i<Type::TypeSize(); ++i)  {
    n = Type::TypeAt(i).Name(SCOPED);
    cout << i << ": " << n << endl;
  }
}

/// Access to reflection information by type name
DbReflex::TypeH DbReflex::forTypeName(const string& name)  {
  // cout << "<-> DbReflex::forTypeName(" << name << ")" << endl;
  Type t = Type::ByName(name);
  return t;
}

/// Access to reflection information by type info
DbReflex::TypeH DbReflex::forTypeInfo(const type_info& info)  {
  Type t = Type::ByTypeInfo(info);
  return (t)? t : forTypeName( DbTypeInfo::typeName(info) );
}

string DbReflex::fullTypeName(DbReflex::TypeH& typ)  {
  return typ.Name(SCOPED);
}

/// Convert Guid to nomalized string (all char upper case!)
string DbReflex::guidString(const Guid& id)  {
  string s = id.toString();
  normalizeGuidString(s);
  return s;
}

/// Convert Guid to nomalized string (all char upper case!)
Guid DbReflex::guid(DbReflex::TypeH& type)  {
  TypeMap::iterator i = type_mapping().find(type);
  if ( i != type_mapping().end() )  {
    return (*i).second;
  }
  if ( type )  {
    string s;
    PropertyList pl = type.Properties();
    if ( pl.HasProperty("ClassID") )  {
      s = pl.PropertyAsString("ClassID");
      Guid id(s);
      guid_mapping()[id] = type;
      type_mapping()[type] = id;
      return id;
    }
    else  {
      char buff[20];
      genMD5(fullTypeName(type),buff);
      s = ((Guid*)buff)->toString();
      guid_mapping()[*(Guid*)buff] = type;
      type_mapping()[type] = *(Guid*)buff;
      return *(Guid*)buff;
    }
  }
  return Guid::null();
}

/// Convert Guid to nomalized string (all char upper case!)
string DbReflex::guidString(DbReflex::TypeH& type)  {
  return guid(type).toString();
}

/// Access to reflection information by Guid
DbReflex::TypeH DbReflex::forGuid(const Guid& id)  {
  GuidMap& m = guid_mapping();
  GuidMap::iterator k = m.find(id);
  if ( k != m.end() )  {
    return (*k).second;
  }
  // Check the transformation map
  const DbTypeInfo* info = 0;
  if ( DbTransform::getShape(id, info).isSuccess() )  {
    Type t = info->clazz();
    if ( t )  {
      Guid g = guid(t);
      if ( ::memcmp(&g, &id, sizeof(Guid))==0 )  {
        return t;
      }
    }
  }
  // GUID not in the map....refresh the map for freshly loaded classes
  static bool first = true;
  if ( first )  {
    first = false;
    if ( doTrace() )  {
      cout << "WARNING: Transformation <-> Guid map looks incomplete for id:" 
        << id.toString() << endl;
    }
  }
  Type wanted;
  for(size_t j=0, n=Type::TypeSize(); j<n; ++j)  {
    TypeH t = Type::TypeAt(j);
    if ( t.IsClass() || t.IsStruct() )  {
      Guid g = guid(t);
      if ( ::memcmp(&g, &id, sizeof(Guid))==0 )  {
        wanted = t;
      }
    }
  }
  return wanted ? wanted : Type();
}
