// $Id: DbTypeInfo.cpp,v 1.75 2009/01/21 14:25:50 frankb Exp $
//====================================================================
//  DbTypeInfo implementation
//--------------------------------------------------------------------
//
//  Package    : StorageSvc (The POOL project)
//
//  Description: Type information for persistent objects
//
//  @author      M.Frank
//====================================================================

// Framework include files
#include "StorageSvc/DbToken.h"
#include "StorageSvc/DbArray.h"
#include "StorageSvc/DbColumn.h"
#include "StorageSvc/DbReflex.h"
#include "StorageSvc/DbTypeInfo.h"
#include "StorageSvc/DbObjectSet.h"
#include "StorageSvc/DbTransform.h"
#include "StorageSvc/DbInstanceCount.h"
#include "POOLCore/Reference.h"
#include "Reflex/Builder/ReflexBuilder.h"
#include "Reflex/Builder/NamespaceBuilder.h"

#include "DbString.h"

// STL include files
#include <stdexcept>
#include <iostream>
#include <cstring>
#include <cstdio>
#include <map>

#if !defined(WIN32) && !defined(__ICC)
#include "cxxabi.h"
#endif

#include "RVersion.h"
#if ROOT_VERSION_CODE < ROOT_VERSION(5,19,0)
using namespace ROOT;
#endif

using namespace Reflex;
using namespace std;

typedef const vector<void*>& _Args;

#if ROOT_VERSION_CODE < ROOT_VERSION(5,21,3)
template <class T> static void* ctor(void* m, _Args, void*)
{  return ::new(m) T();                                             }
template <class T> static void* dtor(void* m, _Args, void*)
{  ((T*)m)->~T(); return 0;                                         }
static void* Token_db(void* m, _Args, void*)
{  static string p; p=((pool::Token*)m)->dbID();    return &p;      }
static void* Token_cont(void* m, _Args, void*)
{  static string p; p=((pool::Token*)m)->contID();  return &p;      }
static void* Token_technology(void* m, _Args, void*)
{  static int p; p=((pool::Token*)m)->technology();      return &p; }
static void* Token_classID(void* m, _Args, void*)
{  static pool::Guid p; p=((pool::Token*)m)->classID();  return &p; }
static void* Token_setDb(void* m, _Args a, void*)
{  ((pool::Token*)m)->setDb(*(string*)a[0]);        return 0;       }
static void* Token_setCont(void* m, _Args a, void*)
{  ((pool::Token*)m)->setCont(*(string*)a[0]);      return 0;       }
static void* Token_setTechnology(void* m, _Args a, void*)
{  ((pool::Token*)m)->setTechnology(*(int*)a[0]);        return 0;  }
static void* Token_setClassID(void* m, _Args a, void*)
{  ((pool::Token*)m)->setClassID(*(pool::Guid*)a[0]);    return 0;  }

#else

template <class T> static void ctor(void* ret, void* m, _Args, void*)
{  if(ret)*(void**)ret = ::new(m) T(); else ::new(m) T();           }
template <class T> static void dtor(void*, void* m, _Args, void*)  
{  ((T*)m)->~T();                                                   }
static void Token_db(void* ret, void* m, _Args, void*)
{  if(ret) *(new(ret) std::string) = ((pool::Token*)m)->dbID();     }
static void Token_cont(void* ret, void* m, _Args, void*)
{  if(ret) *(new(ret) std::string) = ((pool::Token*)m)->contID();   }
static void Token_technology(void* ret, void* m, _Args, void*)
{  if(ret) *(new(ret) (int)) = ((pool::Token*)m)->technology();     }
static void Token_classID(void* ret, void* m, _Args, void*)
{  if(ret) *(new(ret) pool::Guid()) = ((pool::Token*)m)->classID(); }
static void Token_setDb(void* m, void*, _Args a, void*)
{  ((pool::Token*)m)->setDb(*(string*)a[0]);                        }
static void Token_setCont(void* m, void*, _Args a, void*)
{  ((pool::Token*)m)->setCont(*(string*)a[0]);                      }
static void Token_setTechnology(void* m, void*, _Args a, void*)
{  ((pool::Token*)m)->setTechnology(*(int*)a[0]);                   }
static void Token_setClassID(void* m, void*, _Args a, void*)
{  ((pool::Token*)m)->setClassID(*(pool::Guid*)a[0]);               }

#endif

template <class T, class Q> void __initdbarraydictionary(T*, Q*, const string& typ, const string& clid) {
  Type type_void = TypeBuilder("void");
  Type ptr_type  = TypeDistiller<T*>::Get();
  ClassBuilderT<Q > b;
  b.AddProperty("ClassID", clid);
  b.AddFunctionMember(FunctionTypeBuilder(type_void), typ.c_str(),  ctor<Q >, 0, 0, PUBLIC | CONSTRUCTOR );
  b.AddFunctionMember(FunctionTypeBuilder(type_void), string("~"+typ).c_str(), dtor<Q >, 0, 0, PUBLIC | DESTRUCTOR | VIRTUAL );
  b.AddDataMember(TypeDistiller<int>::Get(), "m_size", size_t(((char*)&((Q*)64)->m_size)-64), PUBLIC);
  b.AddDataMember(ptr_type, "m_buffer", size_t(((char*)&((Q*)64)->m_buffer)-64), PUBLIC);
  Type t = Type::ByTypeInfo(typeid(Q));
  t.DataMemberByName("m_size").Properties().AddProperty(  "comment", "Blob size specifier");
  t.DataMemberByName("m_buffer").Properties().AddProperty("comment", "[m_size]");
}

#define __INITDBARRAYDICTIONARY(inside, wrapper, clid)   __initdbarraydictionary((inside*)0, (wrapper*)0, #wrapper, clid);

POOL_BEGIN_STATIC_BLOCK(DbTypeInfo)
  using namespace pool;

  // Build pool namespace
  NamespaceBuilder("pool");
  // Build std namespace
  NamespaceBuilder("std");
  NamespaceBuilder("stdext");
  NamespaceBuilder("__gnu_cxx");

  // Dictionary declaration for reference base class
  ClassBuilderT<pool::Reference >()
    .AddProperty("ClassID", "56A400B9-BD5E-4A68-996B-DDFC53350653")
    .AddFunctionMember<void(void)>("Reference",  ctor<Reference>, 0, 0, PUBLIC | CONSTRUCTOR )
    .AddFunctionMember<void(void)>("~Reference", dtor<Reference>, 0, 0, PUBLIC | DESTRUCTOR | VIRTUAL );
  // Dictionary declaration for Token base class
  ClassBuilderT<pool::Token >()
    .AddProperty("ClassID", "E013D4B1-CA9C-4E3C-9BE2-8691BBAFCB9A")
    .AddFunctionMember<void(void)>("Token",  ctor<Token>, 0, 0, PUBLIC | CONSTRUCTOR )
    .AddFunctionMember<void(void)>("~Token", dtor<Token>, 0, 0, PUBLIC | DESTRUCTOR | VIRTUAL )
    .AddFunctionMember<const string&(void)>("db",    Token_db, 0, 0, PUBLIC )
    .AddFunctionMember<void(const string&)>("setDb", Token_setDb, 0, 0, PUBLIC )
    .AddFunctionMember<const string&(void)>("contID",  Token_cont, 0, 0, PUBLIC )
    .AddFunctionMember<void(const string&)>("setCont", Token_setCont, 0, 0, PUBLIC )
    .AddFunctionMember<int(void)>("technology", Token_technology, 0, 0, PUBLIC )
    .AddFunctionMember<void(int)>("setTechnology", Token_setTechnology, 0, 0, PUBLIC )
    .AddFunctionMember<const pool::Guid&(void)>("classID", Token_classID, 0, 0, PUBLIC )
    .AddFunctionMember<void(const pool::Guid&)>("setClassID", Token_setClassID, 0, 0, PUBLIC );

  // Dictionary declaration for DbToken class
  ClassBuilderT<pool::DbToken >()
    .AddProperty("ClassID", "B30D24AA-1F1D-4EE0-A9B6-C10405D65854")
    .AddBase<pool::Token>()
    .AddFunctionMember<void(void)>("DbToken",  ctor<DbToken>, 0, 0, PUBLIC | CONSTRUCTOR )
    .AddFunctionMember<void(void)>("~DbToken", dtor<DbToken>, 0, 0, PUBLIC | DESTRUCTOR | VIRTUAL );

  // Dictionary declaration for external link class
  ClassBuilderT<pool::DbLink > b;
  b.AddProperty("ClassID", "9E944D67-6C88-47E2-9834-FCCDA9020F29");
  b.AddFunctionMember<void(void)>("DbLink",  ctor<DbLink>, 0, 0, PUBLIC | CONSTRUCTOR );
  b.AddFunctionMember<void(void)>("~DbLink", dtor<DbLink>, 0, 0, PUBLIC | DESTRUCTOR );
  //b.AddDataMember<int>("first",   OffsetOf(pool::DbLink,first));
  //b.AddDataMember<int>("second",  OffsetOf(pool::DbLink,second));
  b.AddDataMember<int>("first",   size_t(((char*)&((pool::DbLink*)64)->first)-64));
  b.AddDataMember<int>("second",  size_t(((char*)&((pool::DbLink*)64)->second)-64));

  // Dictionary declaration for internal Database strings
  ClassBuilderT<pool::DbString >()
    .AddProperty("ClassID", "DA8F479C-09BC-49D4-94BC-99D025A23A3B")
    .AddFunctionMember<void(void)>("DbString",  ctor<DbString>, 0, 0, PUBLIC | CONSTRUCTOR )
    .AddFunctionMember<void(void)>("~DbString", dtor<DbString>, 0, 0, PUBLIC | DESTRUCTOR | VIRTUAL );

  __INITDBARRAYDICTIONARY(char,          CharDbArray,  "42A3E929-5016-4D97-932A-15D21E2D17A9");
  __INITDBARRAYDICTIONARY(unsigned char, UCharDbArray, "B798D007-C1B2-41AD-8BEC-ABC6F0E342E8");
  __INITDBARRAYDICTIONARY(short,         ShortDbArray, "35A281C2-8A14-430A-BEDC-FC8CA5C88B4B");
  __INITDBARRAYDICTIONARY(unsigned short,UShortDbArray,"A26688E9-C6A2-4F31-B954-F7B992A3C638");
  __INITDBARRAYDICTIONARY(int,           IntDbArray,   "2C02D4FD-3AC8-4241-A6A2-63CD2C86478C");
  __INITDBARRAYDICTIONARY(unsigned int,  UIntDbArray,  "F297366F-A876-4FCE-9AC3-D4BEDAEAFAE8");
  __INITDBARRAYDICTIONARY(long,          LongDbArray,  "1601B72A-87FE-45CF-B128-FE577C0A438C");
  __INITDBARRAYDICTIONARY(unsigned long, ULongDbArray, "B228247A-AFEE-449E-BD6D-3A1C43555662");
  __INITDBARRAYDICTIONARY(float,         FloatDbArray, "EC636D76-EE64-431A-A046-BEE7A3B73456");
  __INITDBARRAYDICTIONARY(double,        DoubleDbArray,"DF9DDD4F-CBFC-4b75-8A46-FF0A64DD0B2F");

POOL_END_STATIC_BLOCK(DbTypeInfo)

static const string __typeinfoName( const type_info& tinfo) {
  const char* class_name = tinfo.name();
  string result;
#ifdef WIN32
  size_t off = 0;
  if ( ::strncmp(class_name, "class ", 6) == 0 )   {
    // The returned name is prefixed with "class "
    off = 6;
  }
  if ( ::strncmp(class_name, "struct ", 7) == 0 )   {
    // The returned name is prefixed with "struct "
    off = 7;
  }
  if ( off != string::npos )    {
    string tmp = class_name + off;
    size_t loc = 0;
    while( (loc = tmp.find("class ")) != string::npos )  {
      tmp.erase(loc, 6);
    }
    loc = 0;
    while( (loc = tmp.find("struct ")) != string::npos )  {
      tmp.erase(loc, 7);
    }
    result = tmp;
  }
  else  {
    result = class_name;
  }
  // Change any " *" to "*"
  while ( (off=result.find(" *")) != string::npos )    {
    result.replace(off, 2, "*");
  }
  // Change any " &" to "&"
  while ( (off=result.find(" &")) != string::npos )    {
    result.replace(off, 2, "&");
  }
#elif defined(sun)
  result = class_name;
#elif !defined(__ICC)
  if ( ::strlen(class_name) == 1 ) {
    // See http://www.realitydiluted.com/mirrors/reality.sgi.com/dehnert_engr/cxx/abi.pdf
    // for details
    switch(class_name[0]) {
    case 'v':
      result = "void";
      break;
    case 'w':
      result = "wchar_t";
      break;
    case 'b':
      result = "bool";
      break;
    case 'c':
      result = "char";
      break;
    case 'h':
      result = "unsigned char";
      break;
    case 's':
      result = "short";
      break;
    case 't':
      result = "unsigned short";
      break;
    case 'i':
      result = "int";
      break;
    case 'j':
      result = "unsigned int";
      break;
    case 'l':
      result = "long";
      break;
    case 'm':
      result = "unsigned long";
      break;
    case 'x':
      result = "long long";
      break;
    case 'y':
      result = "unsigned long long";
      break;
    case 'n':
      result = "__int128";
      break;
    case 'o':
      result = "unsigned __int128";
      break;
    case 'f':
      result = "float";
      break;
    case 'd':
      result = "double";
      break;
    case 'e':
      result = "long double";
      break;
    case 'g':
      result = "__float128";
      break;
    case 'z':
      result = "ellipsis";
      break;
    }
  }
  else  {
    char buff[16*1024];
    size_t len = sizeof(buff);
    int    status = 0;
    result = __cxxabiv1::__cxa_demangle(class_name, buff, &len, &status);
  }
#else
  result = class_name;
  throw runtime_error("CXXABI is missing for ICC!");
#endif
  return result;
}

namespace pool  {
  const string typeName(const type_info& typ) {
    return __typeinfoName(typ);
  }
  static void delete_DbTypeInfo(void* p) {
    if ( p ) ((pool::DbTypeInfo*)p)->destroy();
  }
  void clearColumns(DbTypeInfo::Columns& cols)  {
    if ( cols.empty() ) return;
    for(DbTypeInfo::Columns::iterator i=cols.begin(); i != cols.end(); ++i)  {
      if ( 0 != (*i) ) delete (*i);
    }
  }
}

using namespace pool;

static DbObjectSet<DbTypeInfo>& typeInfos()   {
  static DbObjectSet<DbTypeInfo> s_typeInfos(delete_DbTypeInfo);
  return s_typeInfos;
}

/// Standard Constructor
DbTypeInfo::DbTypeInfo(const Guid& guid, TypeH cl)
: Shape(),
  m_refCount( 0 ),
  m_columns(),
  m_mult( 0 ),
  m_class( cl )
{
  typeInfos().add(this);
  DbInstanceCount::increment(this);
  m_id     = guid;
}

// Destructor
DbTypeInfo::~DbTypeInfo()    {
  DbInstanceCount::decrement(this);
  typeInfos().remove(this);
  for(size_t i = 0; i < m_columns.size(); ++i )   {
    delete m_columns[i];
  }
  m_columns.clear();
}

/// Access to reflection class (if availible)
DbTypeInfo::TypeH DbTypeInfo::clazz()  const  {
  if ( m_class )  {
    return m_class;
  }
  else if ( !m_class && m_columns.size() > 1 )  {
    static bool convert = false;
    if ( !convert )  {
      try   {
        DbTypeInfo* thisPtr = (DbTypeInfo*)this;
        convert = true;
        thisPtr->m_class = DbReflex::forGuid(m_id);
      }
      catch(...)  {
      }
      convert = false;
    }
    return m_class;
  }
  else if ( !m_class && m_columns.size() == 1 )  {
    DbTypeInfo* thisPtr = (DbTypeInfo*)this;
    thisPtr->m_class = m_columns[0]->type();
    return m_class;
  }
  return m_class;
}

/// Allow usage of base classes
DbStatus DbTypeInfo::declareBase(const DbTypeInfo* pInfo) {
  if ( pInfo )    {
    for(size_t i = 0; i < pInfo->m_columns.size(); ++i )   {
      m_columns.push_back(new DbColumn(*(pInfo->m_columns[i])));
    }
    return Success;
  }
  return Error;
}

/// Create persistable description
const string DbTypeInfo::toString() const   {
  string rep;
  char txt[256];
  rep = "{ID=";
  rep += m_id.toString();
  rep += "}";
  if ( !m_class )
    rep += "{CL=<no_class>}";
  else
    rep += "{CL=" + DbReflex::fullTypeName(m_class) + "}";
  sprintf(txt,"{NCOL=%ld}{CNT=%d}", long(m_columns.size()), m_mult);
  rep += txt;
  for(Columns::const_iterator i=m_columns.begin(); i<m_columns.end();++i) {
    rep += "{COL={";
    rep += (*i)->toString();
    rep += "}}";
  }
  return rep;
}

static const char* itm[][2] = { 
  {"{ID=","}"}, 
  {"{CL=","}"}, 
  {"{NCOL=","}"},
  {"{COL={","}}"} 
};

DbStatus DbTypeInfo::i_fromString( const string& string_rep)  {
  size_t i;
  int ncol=-1;
  string tmp = string_rep;
  string cl_name;
  m_mult = 0;
  for(i = 0; i < m_columns.size(); ++i)
    delete m_columns[i];
  m_columns.clear();
  char* p1 = (char*)tmp.c_str();
  DbColumn* col = 0;
  m_id = Guid::null();
  for(i = 0; i < sizeof(itm)/sizeof(itm[0]); ++i)   {
Again:
    p1 = ::strstr(p1, itm[i][0]);
    if ( p1 )    {
      char* pp1 = p1+strlen(itm[i][0]);
      char* p2 = ::strstr(pp1, itm[i][1]);
      if ( p2 )   {
        p2 += ::strlen(itm[i][1])-1;
        *p1 = 0;
        *p2 = 0;
        switch(i) {
        case 0:
          m_id.fromString(pp1);
          // this->clazz();
          break;
        case 1:
          m_class = DbReflex::forTypeName(pp1);
          if ( m_class && m_id == Guid::null() )  {
            m_id = DbReflex::guid(m_class);
          }
          break;
        case 2:
          ::sscanf(pp1, "%d", &ncol);
          break;
        case 3:
          m_columns.push_back(col=new DbColumn());
          col->fromString(pp1);
          break;
        default:
          break;
        }
        *p1 = '{';
        *p2 = '}';
        if ( i > 2 && int(m_columns.size()) < ncol )  {
          p1++;
          goto Again;
        }
      }
    }
  }  
  return int(m_columns.size()) == ncol ? Success : Error;
}

/// Access type name by type identifier from RTTI
const string DbTypeInfo::typeName(const type_info& typ) {
  return pool::typeName(typ);
}

/// Load type information object from string representation
const DbTypeInfo* DbTypeInfo::fromString(const string& string_rep) {
  Guid clsid(Guid::null());
  DbTypeInfo* pType = new DbTypeInfo(clsid, 0);
  DbStatus sc = pType->i_fromString(string_rep);
  if ( sc.isSuccess() )   {
    DbTypeInfo *typ_info = 0;
    if (DbTransform::getShape(pType->shapeID(), (const DbTypeInfo*&)typ_info) == DbStatus::Success) {
      delete pType;
      return typ_info;
    }
    DbTransform::regShape(pType);
    return pType;
  }
  delete pType;
  return 0;
}

/// Create type information using Guid only Class must already be registered.
DbTypeInfo* DbTypeInfo::create(const string& cl_name)  {
  Columns cols;
  return create(cl_name, cols);
}

/// Create type information using Guid only Class must already be registered.
DbTypeInfo* DbTypeInfo::create(const Guid& guid)  {
  Columns cols;
  return create(guid, cols);
}

/// Create type information using Guid only Class must already be registered.
DbTypeInfo* DbTypeInfo::createEx(const Guid& guid)  {
  Columns cols;
  return createEx(guid, cols);
}

/// Create type information using name
DbTypeInfo* DbTypeInfo::create(const string& cl_name, Columns& cols)   {
  TypeH type = DbReflex::forTypeName(cl_name);
  if ( type )    {
    Guid guid(DbReflex::guid(type));
    DbTypeInfo* typ_info = 0;
    if (DbTransform::getShape(guid, (const DbTypeInfo*&)typ_info) == DbStatus::Success) {
       clearColumns(cols);
       return typ_info;
    }
    typ_info = new DbTypeInfo(guid, type);
    if ( cols.size() == 0 )   {
      string full = DbReflex::fullTypeName(type);
      string nam = (full.substr(0,2)=="::") ? full.substr(2) : full;
      typ_info->m_columns.clear();
      typ_info->m_columns.push_back(new DbColumn(type.Name(),
                                    nam,
                                    DbColumn::POINTER,
                                    0));
    }
    else  {
      typ_info->m_columns = cols;
      cols.clear();
    }
    DbTransform::regShape(typ_info);
    return typ_info;
  }
  clearColumns(cols);
  return 0;
}

/// Create type information using Guid only Class must already be registered.
DbTypeInfo* DbTypeInfo::create(const Guid& guid, Columns& cols)   {
  DbTypeInfo* typ_info = 0;
  if (DbTransform::getShape(guid, (const DbTypeInfo*&)typ_info) == DbStatus::Success) {
    clearColumns(cols);
    return typ_info;
  }
  TypeH cl = DbReflex::forGuid(guid);
  if ( cl )    {
    typ_info = new DbTypeInfo(guid, cl);
    if ( cols.size() == 0 )   {
      string full = DbReflex::fullTypeName(cl);
      string nam = (full.substr(0,2)=="::") ? full.substr(2) : full;
      vector<const DbColumn*> c;
      c.push_back(new DbColumn(cl.Name(),
                               nam,
                               DbColumn::POINTER,
                               0));
      typ_info->m_columns = c;
      c.clear();
    }
    else  {
      typ_info->m_columns = cols;
      cols.clear();
    }
    DbTransform::regShape(typ_info);
    return typ_info;
  }
  clearColumns(cols);
  return 0;
}

/// Create type information using Guid only Class must already be registered.
DbTypeInfo* DbTypeInfo::createEx(const Guid& guid, Columns& cols)   {
  DbTypeInfo* typ_info = 0;
  if (DbTransform::getShape(guid, (const DbTypeInfo*&)typ_info) == DbStatus::Success) {
    clearColumns(cols);
    return typ_info;
  }
  TypeH cl = DbReflex::forGuid(guid);
  if ( !(cols.size() == 0 && !cl) ) {
    typ_info = new DbTypeInfo(guid, cl);
    if ( cols.size() == 0 ) {
      string full = DbReflex::fullTypeName(cl);
      string nam = (full.substr(0,2)=="::") ? full.substr(2) : full;
      vector<const DbColumn*> c;
      c.push_back(new DbColumn(cl.Name(),
                                nam,
                                DbColumn::POINTER,
                                0));
      typ_info->m_columns = c;
      c.clear();
    }
    else {
      typ_info->m_columns = cols;
      cols.clear();
    }
    DbTransform::regShape(typ_info);
    return typ_info;
  }
  clearColumns(cols);
  return 0;
}

/// Delete reference to object
void DbTypeInfo::deleteRef() const {
  if (--m_refCount <= 0) {
    DbTransform::removeShape(this);
    delete this;
  }
}
