// $Id: poolDb.cpp,v 1.31 2008/10/21 08:40:00 frankb Exp $
//====================================================================
//  DbFileFactory implementation
//--------------------------------------------------------------------
//
//  Package    : System (The POOL project)
//
//  Description: Management of the open Db files
//
//  @author      M.Frank
//====================================================================

#include "StorageSvc/pool.h"
#include "StorageSvc/DbType.h"
#include <stdexcept>
#include <iostream>
#include <iomanip>
#include <cstdio>
#include <map>
using namespace std;

static bool s_debug = (0 != ::getenv("POOL_TRACE")) || (0 != ::getenv("POOLDB_TRACE"));

static void startDebugger()  {
#ifdef WIN32
    _asm int 3
#endif
}

bool pool::doTrace() {
  return s_debug;
}

void pool::enableBreak(bool yes_no)   {
  s_debug = yes_no;
}

/// Debug break with printout and exception chaining
void pool::debugBreak( const string& src, 
                       const string& msg,
                       const exception& e,
                       bool rethrow)
{
  string m = "Unknown Source> ";
  m += e.what();
  m += "\n";
  m += src + "> " + msg;
  cout << "ERROR (pool): " << endl << m << endl;
  debugBreak();
  if ( rethrow )    {
    throw runtime_error(m);
  }
}

void pool::debugBreak(const string& src, const string& msg, bool rethrow) {
  string m = src + "> " + msg;
  cout << "ERROR (pool): " << endl << m << endl;
  debugBreak();
  if ( rethrow )    {
    throw runtime_error(m);
  }
}

void pool::debugBreak()   {
  if ( s_debug )    {
    startDebugger();
  }
  else  {
    //cout << "debugbreak (poolDb) However, breaks are DISABLED" 
    //          << endl;
  }
}

/// Translate acce mode to string
const char* pool::accessMode(pool::DbAccessMode mode)   {
  if      ( mode & pool::READ      ) 
    return "READ     ";
  else if ( mode & pool::UPDATE && mode & pool::CREATE )
    return "CREA/UPDA";
  else if ( mode & pool::UPDATE    ) 
    return "UPDATE   ";
  else if ( mode & pool::CREATE    )
    return "CREATE   ";
  else if ( mode & pool::NOT_OPEN  ) 
    return "NOT_OPEN ";
  else                                 
    return "UNKNOWN  ";
}

/// Bad pointer assignment occured. Do printout for debugging puposes
void pool::badAssignMessage(const char* msg)   {
  cout << "ERROR (pool): "
            << "--> Invalid pointer assignment "
            << msg << endl;
}

/// Last resort error handler. Throws exception if sc.severity==ERROR
void pool::unexpectedProblem(const DbStatus& sc, const char* msg) {
  cout << "ERROR (pool): " << msg << endl;
  switch(sc.severity()) {
  case DbStatus::Info:
  case DbStatus::Success:
  case DbStatus::Warning:
    break;
  case DbStatus::Error:
    /// Severe problem: throw exception
    break;
  }
}

