#include "TestApp.h"

#include "../../src/XPFN.h"
#include "../../src/XLFN.h"

#include "FileCatalog/IFileCatalogImpl.h"
#include "FileCatalog/IFileCatalogEditorImpl.h"
#include "FileCatalog/FCException.h"

#include "POOLCore/Guid.h"

#include <stdexcept>

TestApp::TestApp():
  TestBase()
{}


TestApp::~TestApp()
{}


void
TestApp::run()
{
  const std::string dbService = "file:test.xml";
  this->resetCatalog( dbService );
  pool::IFileCatalogImpl* catalog = this->retrieveCatalog( dbService );
  catalog->connect( pool::FC::Update );
  catalog->start();

  // Fill-in a few entries
  pool::xmlfc::XPFN pfn;

  std::string pf = "pfn1";
  std::string type = "ROOT_All";
  pool::Guid guid;
  pool::Guid::create(guid);

  pfn.setValues( pf, guid.toString(), type );
  catalog->editor().insertPFN( pfn );

  pf = "pfn1_2";
  pfn.setValues( pf, guid.toString(), type );
  catalog->editor().insertPFN( pfn );

  pf = "pfn2";
  pool::Guid::create(guid);
  pfn.setValues( pf, guid.toString(), type );
  catalog->editor().insertPFN( pfn );

  pf = "pfn1";
  pfn.setValues( pf, guid.toString(), type );
  bool exceptionCaught = false;
  try {
    catalog->editor().insertPFN( pfn );
  }
  catch( pool::FCduplicatePFNException& ) {
    exceptionCaught = true;
  }

  if ( ! exceptionCaught ) {
    delete catalog;
    throw std::runtime_error( "Did not throw FCduplicatePFNException" );
  }

  pool::xmlfc::XLFN lfn;
  std::string lf = "lf1";
  lfn.setValues( lf, guid.toString() );
  catalog->editor().insertLFN( lfn );
  lf = "lf2";
  lfn.setValues( lf, guid.toString() );
  catalog->editor().insertLFN( lfn );

  exceptionCaught = false;
  try {
    catalog->editor().insertLFN( lfn );
  }
  catch( pool::FCduplicateLFNException& ) {
    exceptionCaught = true;
  }

  if ( ! exceptionCaught ) {
    delete catalog;
    throw std::runtime_error( "Did not throw FCduplicateLFNException" );
  }

  catalog->commit();
  catalog->disconnect();
  delete catalog;
}
